/*
 * TulipChain is the dmoz browser.	http://ostermiller.org/tulipchain/
 * Copyright (C) 2002-2003 Stephen Ostermiller
 * http://ostermiller.org/contact.pl?regarding=TulipChain
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * See COPYING.TXT for details.
 */

package com.Ostermiller.TulipChain;

import java.io.*;
import java.net.*;
import java.util.*;

public class AutoCorrector implements Runnable, Haltable, DmozEditTask {

	DmozListing listing;

	public AutoCorrector(DmozListing listing){
		this.listing = listing;
	}

	public Thread getThread(){
		return correctThread;
	}

	private Thread correctThread = null;

	public void start() {
		if (correctThread == null || !correctThread.isAlive()) {
			correctThread = new Thread(this, "AutoCorrect");
			correctThread.start();
		}
	}

	public void run(){
		try {
			halt = false;
			autoCorrect();
		} catch (IOException e){
			System.err.println(e.getMessage());
		}
	}

	private volatile boolean halt = false;
	public void halt(){
		halt = true;
		getThread().interrupt();
	}

	public String getCatName(){
		return listing.getParent().getName();
	}

	public void autoCorrect() throws IOException {
		try {
			URLStatus s = listing.getURLwStatus();
			DmozCat parent = listing.getParent();
			String catName = parent.getName();
			String reviewedName = null;
			if (catName.endsWith("/unreviewed /")){
				reviewedName = catName.substring(0, catName.length()-12);
			}
			if (s.canAutoCorrect()){
				URL url = null;
				if (reviewedName != null){
					url = new URL("http://" + TulipChain.getEditingHost() + "/editors/editunrev2.cgi");
				} else {
					url = new URL("http://" + TulipChain.getEditingHost() + "/editors/editurl2.cgi");
				}
				HttpURLConnection connection = (HttpURLConnection)(url.openConnection());
				TulipChain.setAgent(connection);
				TulipChain.setDmozAuthentication(connection);
				connection.setDoOutput(true);
				connection.setInstanceFollowRedirects(false);
				StringBuffer sb = new StringBuffer();
				if (reviewedName != null){
					sb.append(URLEncoder.encode("site", parent.getEncoding()));
					sb.append("=");
					sb.append(URLEncoder.encode(listing.getIDNum(), parent.getEncoding()));
					sb.append("&");
					sb.append(URLEncoder.encode("cat", parent.getEncoding()));
					sb.append("=");
					sb.append(URLEncoder.encode(reviewedName, parent.getEncoding()));
					sb.append("&");
					sb.append(URLEncoder.encode("typecat", parent.getEncoding()));
					sb.append("=");
					sb.append(URLEncoder.encode(reviewedName, parent.getEncoding()));
				} else {
					sb.append(URLEncoder.encode("cat", parent.getEncoding()));
					sb.append("=");
					sb.append(URLEncoder.encode(catName, parent.getEncoding()));
				}
				String host = s.getRedirectURL().getHost();
				if (catName.startsWith("Regional/Europe/United_Kingdom/") || (host != null && (host.endsWith(".uk") || host.endsWith(".UK")))){
					sb.append("&uksite=on");
				}
				sb.append("&");
				sb.append(URLEncoder.encode("url", parent.getEncoding()));
				sb.append("=");
				sb.append(URLEncoder.encode(s.url, parent.getEncoding()));
				sb.append("&");
				sb.append(URLEncoder.encode("newurl", parent.getEncoding()));
				sb.append("=");
				sb.append(URLEncoder.encode(s.getRedirectURL().toString(), parent.getEncoding()));
				sb.append("&");
				sb.append(URLEncoder.encode("newtitle", parent.getEncoding()));
				sb.append("=");
				sb.append(URLEncoder.encode(listing.getTitle(), parent.getEncoding()));
				sb.append("&");
				sb.append(URLEncoder.encode("newdesc", parent.getEncoding()));
				sb.append("=");
				sb.append(URLEncoder.encode(listing.getDescription(), parent.getEncoding()));
				sb.append("&");
				sb.append(URLEncoder.encode("newnote", parent.getEncoding()));
				sb.append("=");
				sb.append(
					URLEncoder.encode(
						"TulipChain/" +
						TulipChain.version +
						" Auto Correct " +
						s.getDisplayStatus(),
						parent.getEncoding()
					)
				);
				if (listing.getMediaDate() != null){
					sb.append("&");
					sb.append(URLEncoder.encode("date", parent.getEncoding()));
					sb.append("=");
					sb.append(
						URLEncoder.encode(
							TulipChain.mediaDateSubmitFormat.format(listing.getMediaDate()),
							parent.getEncoding()
						)
					);
				}
				if (listing.getAgeTagKids()){
					sb.append("&");
					sb.append(URLEncoder.encode("age-kids", parent.getEncoding()));
					sb.append("=");
					sb.append(URLEncoder.encode("on", parent.getEncoding()));
				}
				if (listing.getAgeTagTeens()){
					sb.append("&");
					sb.append(URLEncoder.encode("age-teen", parent.getEncoding()));
					sb.append("=");
					sb.append(URLEncoder.encode("on", parent.getEncoding()));
				}
				if (listing.getAgeTagMatureTeens()){
					sb.append("&");
					sb.append(URLEncoder.encode("age-mteen", parent.getEncoding()));
					sb.append("=");
					sb.append(URLEncoder.encode("on", parent.getEncoding()));
				}
				sb.append("&");
				sb.append(URLEncoder.encode("omit", parent.getEncoding()));
				sb.append("=");
				if (reviewedName != null){
					sb.append(URLEncoder.encode("unrev", parent.getEncoding()));
				} else {
					sb.append(URLEncoder.encode("no", parent.getEncoding()));
				}
				sb.append("&");
				sb.append(URLEncoder.encode("submit", parent.getEncoding()));
				sb.append("=");
				sb.append(URLEncoder.encode("Update", parent.getEncoding()));
				OutputStream out = connection.getOutputStream();
				out.write(sb.toString().getBytes());
				out.write("\r\n\r\n".getBytes());
				out.close();
				InputStream in = connection.getInputStream();
				int read;
				while ((read = in.read()) != -1) {
				}
				in.close();
				listing.autoCorrectURL();
			}
			correctThread = null;
		} catch (UnsupportedEncodingException uee){
		}
	}
}
