/*
 * TulipChain is the dmoz browser.	http://ostermiller.org/tulipchain/
 * Copyright (C) 2001-2002 Stephen Ostermiller
 * http://ostermiller.org/contact.pl?regarding=TulipChain
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * See COPYING.TXT for details.
 */

package com.Ostermiller.TulipChain;

import javax.swing.*;
import javax.swing.table.*;
import javax.swing.event.*;
import javax.swing.tree.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.net.*;
import java.util.*;
import java.util.regex.*;

public class CatSelecter implements Runnable, Haltable {
	private String cat;

	public CatSelecter(String cat){
    	if (cat.equals(TulipChain.labels.getString("base_category"))){
        	cat = "/";
        }
		this.cat = cat;
	}

	public Thread getThread(){
		return loadThread;
	}

	private Thread loadThread = null;

	public void start() {
		if (loadThread == null || !loadThread.isAlive()) {
			loadThread = new Thread(this, "CatLoader");
			loadThread.start();
		}
	}

	public void run(){
		try {
			halt = false;
			selectCat(cat);
		} catch (IOException e){
			System.err.println(e.getMessage());
		}
	}

	private volatile boolean halt = false;
	public void halt(){
		halt = true;
		getThread().interrupt();
	}

    private static final Pattern DMOZ_URL = Pattern.compile("^(?i:http\\://)?(?:(?:^\\/)+\\.)?(?i:dmoz.org)(?:\\:(?:[0-9]+))?/(.*)$");

	public void selectCat(String cat) throws IOException{
		cat = cat.trim();
        Matcher matcher = DMOZ_URL.matcher(cat);
        if (matcher.matches()){
        	System.out.println(cat + " " + matcher.group(1));
        	cat = matcher.group(1);
        }
		if (cat.startsWith("/")){
			cat = cat.substring(1, cat.length());
		}
		if (cat.toLowerCase().startsWith("editors/")){
			int start = cat.indexOf("cat=");
			if (start != -1){
				int end = cat.indexOf("&", start);
				if (end == -1){
					end = cat.length();
				}
				if (start+4 < end){
					cat = cat.substring(start+4, end);
				}
			}
		}
		if (!cat.endsWith("/")){
			cat = cat + "/";
		}
		boolean unreviewed = cat.endsWith("/unreviewed /");
		if (unreviewed){
			cat = cat.substring(0, cat.length()-12);
		}
		if (cat.indexOf(":") != -1){
			StringBuffer newCat = new StringBuffer(cat.length());
			StringTokenizer tok = new StringTokenizer(cat, ":/", false);
			while (tok.hasMoreTokens()){
				newCat.append(tok.nextToken().trim().replace(' ', '_')).append('/');
			}
			cat = newCat.toString();
		}
		if (cat.toLowerCase().startsWith("top/")){
			cat = cat.substring(4);
		}
		cat = cat.replace(' ', '_');
		if (unreviewed){
			cat += "unreviewed /";
		}
		StringTokenizer tok = new StringTokenizer(cat, "/", false);
		DmozCat category = TulipChain.topCat;
		//find the cat if we already have it
		boolean found = true;
		String name = "";
		while (found && (tok.hasMoreTokens())){
			if (halt){
				return;
			}
			name += tok.nextToken() + "/";
			found = false;
			for (int i=0; i<category.getChildCount() && !found; i++){
				DmozCat nextCat = (DmozCat)category.getChildAt(i);
				if (nextCat.getName().equals(name)){
					if (nextCat.getLastUpdateTime() == null){
						nextCat.load();
					}
					category = nextCat;
					found = true;
				}
			}
			for (int i=0; i<category.getChildCount() && !found; i++){
				DmozCat nextCat = (DmozCat)category.getChildAt(i);
				if (nextCat.getName().toLowerCase().equals(name.toLowerCase())){
					if (nextCat.getLastUpdateTime() == null){
						nextCat.load();
					}
					category = nextCat;
					found = true;
				}
			}
		}
		if (found){
			synchronized (TulipChain.tree){
				TulipChain.lastCatSelect = category;
				TreePath path = new TreePath(category.getPath());
				TulipChain.tree.expandPath(path);
				TulipChain.tree.setSelectionPath(path);
				TulipChain.tree.scrollPathToVisible(path);
			}
		} else {
			throw new IOException(cat + " not found");
		}
	}
}
