/*
 * TulipChain is the dmoz browser.	http://ostermiller.org/tulipchain/
 * Copyright (C) 2001-2002 Stephen Ostermiller
 * http://ostermiller.org/contact.pl?regarding=TulipChain
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * See COPYING.TXT for details.
 */

package com.Ostermiller.TulipChain;

import java.net.*;
import javax.swing.*;
import javax.swing.table.*;
import javax.swing.tree.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.io.*;
import java.text.*;
import HTTPClient.*;

public class DmozCat extends DefaultMutableTreeNode implements DmozEntry, Runnable, Haltable {

	private String catName;
	private Vector sites = new Vector();
	private Date lastUpdate = null;
	private TulipChain tulip = null;
	private boolean spider = false;
	private boolean spidered = false;
	private String encoding;

	private volatile boolean halt = false;
	public void halt(){
		halt = true;
		Thread thread = getThread();
		if (connection != null) connection.stop();
		if (thread != null && thread.isAlive()) thread.interrupt();
		if (connection != null) connection.stop();
		if (thread != null && thread.isAlive()) thread.interrupt();
		if (connection != null) connection.stop();
	}

	public void setTulip(TulipChain tulip){
		this.tulip = tulip;
		Enumeration enum = children();
		while (enum.hasMoreElements()){
			((DmozCat)enum.nextElement()).setTulip(tulip);
		}
	}

	public void sort(Comparator c){
		synchronized(TulipChain.table){
			DmozListing[] listings = getSelected();
			Collections.sort(sites, c);
			alertTableChange(listings);
		}
	}

	public void setSpider(boolean spider){
		this.spider = spider;
	}

	public boolean spidered(){
		return spidered;
	}

	public void setParent(DmozCat cat){
		parent = cat;
	}

	public DmozCat(String catName){
		this.catName = catName;
	}

	public static void main(String[] args){
		try {
			DmozCat cat = new DmozCat("fun");
			cat.encoding="UTF-8";
			cat.load(new FileReader("stuff/cat.html"));
			cat.checkAll();
		} catch (Exception x){
		}
	}

	public DmozCat(String[] subCats){
		this(TulipChain.labels.getString("base_category"));
		for (int i=0; i<subCats.length; i++){
			DmozCat cat = new DmozCat(subCats[i]);
			insert((MutableTreeNode)cat, i);
		}
	}

	public Date getLastUpdateTime(){
		return lastUpdate;
	}

	public void load(String catName) throws IOException {
		this.catName = catName;
		load();
	}

	public void load(Reader in) throws IOException {
		lastUpdate = new Date();
		DmozCatLexer lex = new DmozCatLexer(in);
		while (getChildCount() > 0){
			TulipChain.getTreeModel().removeNodeFromParent((DefaultMutableTreeNode)getLastChild());
		}
		sites.clear();
		DmozEntry tok;
		int i = 0;
		if (!catName.endsWith("/unreviewed /") &&
			tulip.props.getProperty("editor_pass") != null && (tulip.canEdit(catName) || tulip.canGreenbust(catName+"/unreviewed /"))){
			DmozCat unrev = new DmozCat(catName + "unreviewed /");
			unrev.setTulip(tulip);
			TulipChain.getTreeModel().insertNodeInto((MutableTreeNode)unrev, (MutableTreeNode)this, i);
			i++;
		}
		while ((tok = lex.next_token()) != null){
			if (halt){
				return;
			}
			if (tok instanceof DmozCat){
				if (((DmozCat)tok).getName().startsWith(catName)){
					String subCatName = ((DmozCat)tok).getName().substring(catName.length());
					if (subCatName.endsWith("/")) {
						subCatName = subCatName.substring(0, subCatName.length() - 1);
					}
					if (subCatName.length() > 0 && subCatName.indexOf("/") == -1){
						TulipChain.getTreeModel().insertNodeInto((MutableTreeNode)tok, (MutableTreeNode)this, i);
						((DmozCat)tok).setTulip(tulip);
						i++;
					}
				}
			} else if (tok instanceof DmozListing){
				((DmozListing)tok).setParent(this);
				sites.add(tok);
			}
		}
		if (spider){
			 for (Enumeration enum = children() ; enum.hasMoreElements() ;) {
				if (halt) return;
				TulipChain.threadManager.spiderCat((DmozCat)enum.nextElement());
			 }
			 spidered=true;
			 spider=false;
		} else {
			alertTableChange(null);
			spidered = false;
		}
	}

	public void removeSite(DmozListing removeSite){
		removeSites(new DmozListing[]{removeSite});
	}

	public void removeSites(DmozListing[] removeSites){
		synchronized(TulipChain.table){
			DmozListing[] listings = getSelected();
			for (int i=0; i<removeSites.length; i++){
				sites.remove(removeSites[i]);
			}
			alertTableChange(listings);
		}
	}

	public Thread getThread(){
		return loadThread;
	}
	private Thread loadThread = null;
	public void start() {
		if (loadThread == null || !loadThread.isAlive()) {
			loadThread = new Thread(this, "Load");
			loadThread.start();
		}
	}

	public void run(){
		try {
			halt = false;
			load();
		} catch (IOException e){
			System.err.println(e.getMessage());
		}
	}
	
	private HTTPConnection connection;

	public void load() throws IOException {
		try {
			URL url;
			HTTPResponse response;
			Vector headers = new Vector();					
			headers.add(new NVPair("User-Agent", TulipChain.getAgent()));
			String proxyAuthorization = TulipChain.getProxyAUthorization();
			if (proxyAuthorization != null){
				headers.add(new NVPair("Proxy-Authorization", proxyAuthorization));
			}
			if (catName.endsWith("/unreviewed /")){
				String dmozAuthorization = TulipChain.getDmozAuthentication();
				if (dmozAuthorization != null){
					headers.add(new NVPair("Authorization", dmozAuthorization));
				}connection = new HTTPConnection(new URL("http://" + TulipChain.getEditingHost()));
				handleModules(connection);
				String unrevPage = "/editors/editcat-unrev.cgi?cat=" + catName.substring(0, catName.length()-12);
				response = connection.Get(
					unrevPage,
					(NVPair[])null,
					(NVPair[])headers.toArray(new NVPair[0])
				);
			} else {
				boolean loggedIn = TulipChain.props.getProperty("editor_name") != null && !TulipChain.props.getProperty("editor_name").equals("");
				boolean authenticate = (loggedIn && TulipChain.useEditingAsPublicWhenPossible()) || catName.startsWith("Test/");
				String dmozHost;
				if (loggedIn && TulipChain.useEditingAsPublicWhenPossible()){
					dmozHost = TulipChain.getEditingHost();
				} else {
					dmozHost = TulipChain.getPublicHost();
				}
				if (authenticate){
					String dmozAuthorization = TulipChain.getDmozAuthentication();
					if (dmozAuthorization != null){
						headers.add(new NVPair("Authorization", dmozAuthorization));
					}
				}			
				connection = new HTTPConnection(new URL("http://" + dmozHost));
				handleModules(connection);
				String publicPage = "/" + catName;
				response = connection.Get(
					publicPage,
					(NVPair[])null,
					(NVPair[])headers.toArray(new NVPair[0])
				);
			}
			if (response.getStatusCode() != 200){
				throw new IOException("Error from Dmoz server (" + catName + "): " + response.getReasonLine());
			}

			encoding = response.getHeader("charset");
			InputStream in = new BufferedInputStream(response.getInputStream());
			in.mark(2048);
			byte[] buffer = new byte[2048];
			int read = 0;
			int offset = 0;
			while(read != -1 && offset < 2048){
				read = in.read(buffer, offset, 2048-offset);
				if (read != -1) offset += read;
			}
			in.reset();
			String metaEncoding = (new EncodingLexer(new ByteArrayInputStream(buffer))).next_token();
			if (metaEncoding != null){
				encoding = metaEncoding;
			}
			if (encoding==null){
				System.err.println(MessageFormat.format(
					TulipChain.labels.getString("no_encoding"),
					new Object[] {
						catName
					}
				));
				encoding="UTF-8";
			}
	   		load(new InputStreamReader(in, encoding));
		} catch (ModuleException mx){
			throw new IOException (mx.getMessage());
		}
	}
	
	private void handleModules(HTTPConnection connection){
		Class[] modules = connection.getModules();
		for (int moduleIndex=0; moduleIndex<modules.length; moduleIndex++){
			String moduleName = modules[moduleIndex].getName();
			if ("HTTPClient.AuthorizationModule".equals(moduleName)){
				connection.removeModule(modules[moduleIndex]);
			} else {
				//System.out.println(moduleName);
			}
		}
	}

	public String getName(){
		return catName;
	}

	public int getListingCount() {
		return sites.size();
	}

	public DmozListing getListing(int index){
		return ((DmozListing)sites.elementAt(index));
	}

	public String getParentEncoding(){
		if (parent != null && parent instanceof DmozCat){
			return ((DmozCat)parent).getEncoding();
		}
		return "ASCII";
	}

	public String getEncoding(){
		if (encoding == null) return "ASCII";
		return encoding;
	}

	/**
	 * A description of this cat
	 *
	 * @return the name of this cat appropriate for display in a tree view.
	 */
	public String toString(){
		int i = catName.lastIndexOf("/", catName.length()-2);
		String s = catName.replace('_', ' ');
		// this should even work if i == -1
		s = s.substring(i+1, s.length()-1);
		try {
			s = URLDecoder.decode(s, getParentEncoding());
		} catch (java.io.UnsupportedEncodingException uee){
			s = MessageFormat.format(
				TulipChain.labels.getString("unsupported_encoding"),
				new Object[] {
					getParentEncoding()
				}
			);
		}
		return(s);
	}

	/**
	 * Returns true if the receiver is a leaf.
	 *
	 * @return false - All dmoz cats may have sub cats.
	 */
	public boolean isLeaf(){
		return false;
	}

	public void checkAll(){
		checkAll(false);
	}

	public void checkAll(boolean recursive){
		for (int i=0; i<sites.size(); i++){
			if (halt) return;
			TulipChain.threadManager.checkSite((DmozListing)(sites.elementAt(i)));
		}
		if (recursive){
			for (Enumeration enum = children() ; enum.hasMoreElements() ;) {
				if (halt) return;
				((DmozCat)enum.nextElement()).checkAll(true);
			}
		}
	}

	private void alertTableChange(DmozListing[] listings){
		if (TulipChain.table != null && this == TulipChain.tableModel.getCat()){
			synchronized (TulipChain.table){
				if (listings == null) {
					int[] selectedRows = TulipChain.table.getSelectedRows();
					TulipChain.tableModel.fireTableDataChanged();
					for (int i=0; i<selectedRows.length; i++){
						TulipChain.table.addRowSelectionInterval(selectedRows[i], selectedRows[i]);
					}
				} else {
					TulipChain.tableModel.fireTableDataChanged();
					select(listings);
				}
			}
		}
	}

	private void select(DmozListing[] listings){
		if (TulipChain.table != null && this == TulipChain.tableModel.getCat()){
			for (int i=0; i<listings.length; i++){
				int pos = sites.indexOf(listings[i]);
				if (pos != -1) TulipChain.table.addRowSelectionInterval(pos, pos);
			}
		}
	}

	private DmozListing[] getSelected(){
		DmozListing[] listings;
		if (TulipChain.table != null && this == TulipChain.tableModel.getCat()){
			int[] selectedRows = TulipChain.table.getSelectedRows();
			listings = new DmozListing[selectedRows.length];
			for (int i=0; i<selectedRows.length; i++){
				listings[i] = (DmozListing)(sites.elementAt(selectedRows[i]));
			}
		} else {
			listings = new DmozListing[0];
		}
		return listings;
	}
}
