/*
 * DmozCatLexer.java is a generated file.  You probably want to
 * edit DmozCatLexer.lex to make changes.
 */

/*
 * TulipChain is the dmoz browser.	http://ostermiller.org/tulipchain/
 * Copyright (C) 2001-2002 Stephen Ostermiller
 * http://ostermiller.org/contact.pl?regarding=TulipChain
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * See COPYING.TXT for details.
 */

package com.Ostermiller.TulipChain;

import java.io.*;

%%

%public
%class DmozCatLexer
%function next_token
%type DmozEntry
%unicode


%{

	/**
	 * Prints out tokens from a file or System.in.
	 * If no arguments are given, System.in will be used for input.
	 * If more arguments are given, the first argument will be used as
	 * the name of the file to use as input
	 *
	 * @param args program arguments, of which the first is a filename
	 */
	public static void main(String[] args) {
		InputStream in;
		try {
			if (args.length > 0){
				File f = new File(args[0]);
				if (f.exists()){
					if (f.canRead()){
						in = new FileInputStream(f);
					} else {
						throw new IOException("Could not open " + args[0]);
					}
				} else {
					throw new IOException("Could not find " + args[0]);
				}
			} else {
				in = System.in;
			}
			DmozCatLexer shredder;
			try {
				shredder = new DmozCatLexer(new InputStreamReader(in, "ASCII"));
			} catch (UnsupportedEncodingException e){
				System.err.println(e.getMessage());
				shredder = new DmozCatLexer(in);
			}
			DmozEntry t;
			while ((t = shredder.next_token()) != null) {
				System.out.println(t);
			}
		} catch (IOException e){
			System.out.println(e.getMessage());
		}
	}
    
    private String url;
%}


%state SUBCATS
%state SUBCAT
%state SEE_ALSO
%state SITE_LIST
%state END

AnyChar=([^A]|[A])

Blank=([ ])
TAB=([\t])
FF=([\f])
CR=([\r])
LF=([\n])
EOL=({CR}|{LF}|{CR}{LF})
WhiteSpace=({Blank}|{TAB}|{FF}|{EOL})

NormalSubCatStart = ("<hr>"([^<]*"<t"[^<]*)*"<ul>")
AlphaBarStart = ("<center>"{WhiteSpace}*"[")
MonthStart = ("<td valign=top><ul>")

SubCatStart = {NormalSubCatStart}|{AlphaBarStart}|{MonthStart}


%%

<YYINITIAL>  {SubCatStart} {
	yybegin(SUBCATS);
}

<SUBCATS> (("<a"|"<A")([^\"]*)[\"]([^\"]*)) {
    url = yytext().substring(10, yytext().length());
    yybegin(SUBCAT);
}

<SUBCAT> "<\/a>@" {
    /* @link */
	yybegin(SUBCATS);
}

<SUBCAT> "<\/a>" {
    /* subcat */
	yybegin(SUBCATS);
    if (url.endsWith("/")){
    	return(new DmozCat(url));
    }
}

<YYINITIAL, SUBCATS> (("<hr>"{WhiteSpace}*)?"<p>"[^<]*":<ul>") {
	yybegin(SEE_ALSO);
}

<YYINITIAL, SUBCATS, SEE_ALSO> ("<hr>"{WhiteSpace}*"<table border=0>") {
}

<YYINITIAL, SUBCATS, SEE_ALSO> ("<hr>"{WhiteSpace}*"<table cellpadding=0 cellspacing=0>") {
}

<YYINITIAL, SUBCATS, SEE_ALSO> ("<hr>"{WhiteSpace}*"<p><table><tr><td valign=top") {
	// Alt langs when there are no subcats
}

<YYINITIAL, SUBCATS, SEE_ALSO> ("<hr>"{WhiteSpace}*"<ul>") {
	yybegin(SITE_LIST);
}

<SITE_LIST> (("<a"|"<A")([^<]|("<"("a"|"/a"|"b"|"/b"|"img"|"small"|"font"|"/small"|"/font")))*) {
	return new DmozListing(yytext());
}

<YYINITIAL, SITE_LIST, SUBCATS, SEE_ALSO, SUBCAT> ("<"[Hh][Rr]">") {
	yybegin(END);
}

<YYINITIAL, SUBCATS, SITE_LIST, SEE_ALSO, SUBCAT> {AnyChar} {
}

<END> {AnyChar} {
	return(null);
}
