/*
 * TulipChain is the dmoz browser.  http://ostermiller.org/tulipchain/
 * Copyright (C) 2001 Stephen Ostermiller
 * http://ostermiller.org/contact.pl?regarding=TulipChain
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * See COPYING.TXT for details.
 */

package com.Ostermiller.TulipChain;

import java.io.*;
import java.net.*;

public class DmozDeleter implements Runnable, Haltable, DmozEditTask {

	DmozCat cat;
	DmozListing[] listings;
	String message;

	private volatile  boolean halt = false;
	public void halt(){
		halt = true;
		getThread().interrupt();
	}

	public DmozDeleter(DmozCat cat, DmozListing[] listings, String message){
		this.cat = cat;
		this.listings = listings;
		this.message = message;
	}

	public String getCatName(){
		return cat.getName();
	}

	public void run(){
		halt = false;
		delete();
	}

	public void delete(){
		HttpURLConnection connection = null;
		String realCat = cat.getName();
		if (realCat.endsWith("unreviewed /")) {
			realCat = realCat.substring(0, realCat.length()-12);
		}
		try {
			URL url = new URL("http://" + TulipChain.getEditingHost() + "/editors/editcat-unrev2.cgi");
			connection = (HttpURLConnection)url.openConnection();
			TulipChain.setDmozAuthentication(connection);
			connection.setDoOutput(true);
			connection.setInstanceFollowRedirects(false);
			TulipChain.setAgent((HttpURLConnection)connection);
			StringBuffer sb = new StringBuffer();
			sb.append(URLEncoder.encode("cat", "UTF-8"));
			sb.append("=");
			sb.append(URLEncoder.encode(realCat, "UTF-8"));
			sb.append("&");
			for (int i=0; i<listings.length; i++){
				sb.append(listings[i].getIDNum());
				sb.append("=");
				sb.append(URLEncoder.encode("on", "UTF-8"));
				sb.append("&");
				sb.append("cat-" + listings[i].getIDNum());
				sb.append("=");
				sb.append(URLEncoder.encode(realCat, "UTF-8"));
				sb.append("&");
			}
			sb.append(URLEncoder.encode("submit", "UTF-8"));
			sb.append("=");
			sb.append(URLEncoder.encode("Delete Marked Sites", "UTF-8"));
			sb.append("&");
			sb.append(URLEncoder.encode("note", "UTF-8"));
			sb.append("=");
			sb.append(URLEncoder.encode(message + " [TulipChain/" + TulipChain.version + "]", "UTF-8"));
			sb.append("&");
			sb.append(URLEncoder.encode("selcat", "UTF-8"));
			sb.append("=");
			sb.append("&");
			sb.append(URLEncoder.encode("typecat", "UTF-8"));
			sb.append("=");
			OutputStream out = connection.getOutputStream();
			out.write(sb.toString().getBytes());
			out.write("\r\n\r\n".getBytes());
			out.close();

			InputStream in = connection.getInputStream();
			int read;
			// only read once to cut down on time
			// we are throwing these results anyway.
			// dmoz will happily return thousands of listings
			// from a big category here.
			while ((read = in.read()) == 0);
			in.close();
			cat.removeSites(listings);
		} catch (UnsupportedEncodingException uee){
		} catch (MalformedURLException x){
			System.err.println("Malformed URL: " + x.getMessage());
		} catch (IOException x){
			System.err.println("IO Error: " + x.getMessage());
			System.err.println(x);
		}
	}

	private Thread deleteThread = null;
	public void start() {
		if (deleteThread == null || !deleteThread.isAlive()) {
			deleteThread = new Thread(this, "Delete");
			deleteThread.start();
		}
	}

	public Thread getThread(){
		return deleteThread;
	}
}
