/*
 * DmozEntryLexer.java is a generated file.  You probably want to
 * edit DmozEntryLexer.lex to make changes.
 */

/*
 * TulipChain is the dmoz browser.	http://ostermiller.org/tulipchain/
 * Copyright (C) 2001-2002 Stephen Ostermiller
 * http://ostermiller.org/contact.pl?regarding=TulipChain
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * See COPYING.TXT for details.
 */

package com.Ostermiller.TulipChain;

import java.io.*;

%%

%public
%class DmozEntryLexer
%function next_token
%type String

%{

	/**
	 * Prints out tokens from a file or System.in.
	 * If no arguments are given, System.in will be used for input.
	 * If more arguments are given, the first argument will be used as
	 * the name of the file to use as input
	 *
	 * @param args program arguments, of which the first is a filename
	 */
	public static void main(String[] args) {
		InputStream in;
		try {
			if (args.length > 0){
				File f = new File(args[0]);
				if (f.exists()){
					if (f.canRead()){
						in = new FileInputStream(f);
					} else {
						throw new IOException("Could not open " + args[0]);
					}
				} else {
					throw new IOException("Could not find " + args[0]);
				}
			} else {
				in = System.in;
			}
			DmozEntryLexer shredder = new DmozEntryLexer(in);
			String t;
			while ((t = shredder.next_token()) != null) {
				System.out.println(t);
			}
		} catch (IOException e){
			System.out.println(e.getMessage());
		}
	}

	/**
	 * Replace html entities in the string with the real character
	 */
	private String deEntify(String s){
		StringBuffer sb = new StringBuffer();
		int length = s.length();
		for (int i=0; i<length; i++){
			char c = s.charAt(i);
			if (c == '&'){
				if (i+4 < length &&
					s.charAt(i+1) == 'a' &&
					s.charAt(i+2) == 'm' &&
					s.charAt(i+3) == 'p' &&
					s.charAt(i+4) == ';'){
					sb.append('&');
					i+=4;
				} else if (i+5 < length &&
					s.charAt(i+1) == 'q' &&
					s.charAt(i+2) == 'u' &&
					s.charAt(i+3) == 'o' &&
					s.charAt(i+4) == 't' &&
					s.charAt(i+5) == ';'){
					sb.append('"');
					i+=5;
				} else if (i+3 < length &&
					s.charAt(i+1) == 'l' &&
					s.charAt(i+2) == 't' &&
					s.charAt(i+3) == ';'){
					sb.append('<');
					i+=3;
				} else if (i+3 < length &&
					s.charAt(i+1) == 'g' &&
					s.charAt(i+2) == 't' &&
					s.charAt(i+3) == ';'){
					sb.append('>');
					i+=3;
				} else {
					sb.append(c);
				}
			} else {
				sb.append(c);
			}
		}
		return sb.toString();
	}
%}

%unicode
%state URL
%state URLCLEANUP
%state TITLE
%state TITLECLEANUP
%state DESCRIPTION
%state IDNUM
%state IDNUMCLEANUP
%state FINDMEDIADATE
%state MEDIADATE
%state AGETAG
%state END

AnyChar = ([A]|[^A])

BLANK=([ ])
TAB=([\t])
FF=([\f])
CR=([\r])
LF=([\n])
EOL=({CR}|{LF}|{CR}{LF})
WhiteSpace=({BLANK}|{TAB}|{FF}|{EOL})
HTMLWhiteSpace=({WhiteSpace}|"&nbsp;")
HTMLTag=([\<][^\>]*[\>])
DmozMessage=(([\[][^\]]*[\]])|([\(][^\)]*[\)]))
Kids=("Kids")
Teens=("Teens")
MatureTeens=("Mature Teens")
AgeTag=({Kids}|{Teens}|{MatureTeens})
AgeTagBlock=(([\[]({WhiteSpace}*{AgeTag}[\/]?{WhiteSpace}*)*[\]]))

%%


<YYINITIAL> ([^\"]*[\"]) {
	yybegin(URL);
	return(""); // no id number
}

<YYINITIAL> ("<A href=\"/editors/editunrev.cgi?site=") {
	yybegin(IDNUM);
}

<IDNUM> ([0-9]*) {
	yybegin(IDNUMCLEANUP);
	return(yytext());
}

<IDNUMCLEANUP> (([^<]|"<img"|"</a")*[^\"]*[\"]) {
	yybegin(URL);
}

<URL> ([^\"]*) {
	yybegin(URLCLEANUP);
	return(yytext());
}

<URLCLEANUP> ([\"]{WhiteSpace}*([\>])({HTMLTag}|{HTMLWhiteSpace})*) {
	yybegin(TITLE);
}

<TITLE> ([^<]*) {
	yybegin(TITLECLEANUP);
	return(deEntify(yytext()));
}

<TITLECLEANUP> {AgeTagBlock} {
	yybegin(AGETAG);
	return(yytext());
}

<TITLECLEANUP, AGETAG> ({HTMLTag}|{DmozMessage}|{HTMLWhiteSpace}*) {
}

<TITLECLEANUP> (([^\<\-]*)*[-]{HTMLWhiteSpace}*) {
	yybegin(DESCRIPTION);
	return("");
}

<AGETAG> (([^\<\-]*)*[-]{HTMLWhiteSpace}*) {
	yybegin(DESCRIPTION);
}

<DESCRIPTION> ([^\r\n\<]*) {
	yybegin(FINDMEDIADATE);
	return(deEntify(yytext()));
}

<FINDMEDIADATE> "<small>(" {
	yybegin(MEDIADATE);
}

<FINDMEDIADATE> {AnyChar} {
	yybegin(END);
	return("");
}

<MEDIADATE> [^\)\>]* {
	yybegin(END);
	return(yytext());
}

<YYINITIAL, URL, URLCLEANUP, TITLE, TITLECLEANUP, DESCRIPTION, IDNUM, IDNUMCLEANUP, MEDIADATE, END, AGETAG> {AnyChar} {
	return(null);
}

