/*
 * TulipChain is the dmoz browser.	http://ostermiller.org/tulipchain/
 * Copyright (C) 2001 Stephen Ostermiller
 * http://ostermiller.org/contact.pl?regarding=TulipChain
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * See COPYING.TXT for details.
 */

package com.Ostermiller.TulipChain;

import java.io.StringReader;
import java.io.IOException;
import java.util.*;
import java.text.*;
import java.net.*;
import com.Ostermiller.util.StringHelper;

public class DmozListing implements DmozEntry {

	private final static SimpleDateFormat mediaDateInputFormat = new SimpleDateFormat("MMMMM dd, yyyyy");

	private URLStatus url = new URLStatus();
	private String description;
	private String title;
	private String idNum;
	private String mediaDate;
	private boolean ageTagKids = false;
	private boolean ageTagTeens = false;
	private boolean ageTagMatureTeens = false;
	protected DmozCat parent;

	public void setParent(DmozCat cat){
		parent = cat;
		this.url.setParent(parent);
	}

	public DmozListing(String url, String title, String description){
		this(url, title, description, "", "");
	}

	public DmozListing(String url, String title, String description, String idNum){
		this(url, title, description, idNum, "");
	}

	public DmozListing(String url, String title, String description, String idNum, String mediaDate){
		this.idNum = idNum;
		this.url.setListing(this);
		this.url.url = url;
		this.url.setParent(parent);
		this.url.status = 0;
		this.description = description;
		this.title = title;
		this.mediaDate = mediaDate;
		this.ageTagKids = false;
		this.ageTagTeens = false;
		this.ageTagMatureTeens = false;
	}

	public void setAgeTagKids(boolean ageTagKids){
		this.ageTagKids = ageTagKids;
	}

	public boolean getAgeTagKids(){
		return this.ageTagKids;
	}
	public void setAgeTagTeens(boolean ageTagTeens){
		this.ageTagTeens = ageTagTeens;
	}

	public boolean getAgeTagTeens(){
		return this.ageTagTeens;
	}
	public void setAgeTagMatureTeens(boolean ageTagMatureTeens){
		this.ageTagMatureTeens = ageTagMatureTeens;
	}

	public boolean getAgeTagMatureTeens(){
		return this.ageTagMatureTeens;
	}


	public DmozListing(String url, String title, String description, String idNum, String mediaDate,
			boolean ageTagKids, boolean ageTagTeens, boolean ageTagMatureTeens){
		this.idNum = idNum;
		this.url.setListing(this);
		this.url.url = url;
		this.url.setParent(parent);
		this.url.status = 0;
		this.description = description;
		this.title = title;
		this.mediaDate = mediaDate;
		this.ageTagKids = ageTagKids;
		this.ageTagTeens = ageTagTeens;
		this.ageTagMatureTeens = ageTagMatureTeens;
	}

	public DmozListing(String html){
		this.url.setListing(this);
		this.ageTagKids = false;
		this.ageTagTeens = false;
		this.ageTagMatureTeens = false;
		try {
			DmozEntryLexer lex = new DmozEntryLexer(new StringReader(html));
			this.idNum = lex.next_token();
			String nextToken;
			nextToken = lex.next_token();
			this.url.url = StringHelper.unescapeHTML((nextToken==null)?"":nextToken);
			nextToken = lex.next_token();
			this.title = StringHelper.unescapeHTML((nextToken==null)?"":nextToken);
			String ageTags = lex.next_token();
			ageTags = ((ageTags==null)?"":ageTags);
			this.ageTagKids = (ageTags.indexOf("Kids") != -1);
			int teensIndex = ageTags.indexOf("Teens");
			int matureTeensIndex = ageTags.indexOf("Mature Teens");
			this.ageTagTeens = (teensIndex != -1 && (teensIndex < matureTeensIndex || matureTeensIndex == -1));
			this.ageTagMatureTeens = (matureTeensIndex != -1);
			nextToken = lex.next_token();
			this.description = StringHelper.unescapeHTML((nextToken==null)?"":nextToken);
			this.mediaDate = lex.next_token();
		} catch (IOException e){
		}
		if (url.url==null) url.url="";
		if (description==null) description="";
		if (title==null) title="";
		if (mediaDate==null) mediaDate="";
		this.url.status = 0;
		this.url.setParent(parent);
	}

	public String getIDNum(){
		return this.idNum;
	}

	public void setTitle(String title){
		this.title = title;
	}

	public String getTitle(){
		return title;
	}

	public void setMediaDate(String mediaDate){
		this.mediaDate = mediaDate;
	}

	public String getMediaDateString(){
		return mediaDate;
	}

	public Date getMediaDate(){
		if (mediaDate == null || mediaDate.equals("")){
			return null;
		}
		return mediaDateInputFormat.parse(mediaDate, new ParsePosition(0));
	}

	public void setURL(String url){
		this.url.url = url;
		this.url.status = URLStatus.UNKNOWN;
	}

	public void autoCorrectURL(){
		url.autoCorrect();
	}

	public void setDescription(String description){
		this.description = description;
	}

	public String getURL(){
		return url.url;
	}

	public URLStatus getURLwStatus(){
		return url;
	}

	public String getDescription(){
		return description;
	}

	public DmozCat getParent(){
	  return parent;
	}

	public String toString(){
		return (
			"id: " + idNum + "\nurl: " + url + "\ntitle: " + title +
			"\nage tags: " + (ageTagKids?"Kids ":"") + (ageTagTeens?"Teens ":"") + (ageTagMatureTeens?"Mature Teens ":"") +
			"\ndescription: "  + description + "\nmedia date: " + mediaDate
		);
	}

	private static Comparator urlComparator;
	public static Comparator getURLComparator(){
		if (urlComparator == null){
			urlComparator = new Comparator(){
				public int compare(Object o1, Object o2){
					try {
						URL url1 = new URL(((DmozListing)o1).url.url);
						URL url2 = new URL(((DmozListing)o2).url.url);
						String host1 = url1.getHost();
						String host2 = url2.getHost();
						if (host1 == null || host2 == null ||
								host1.length() == 0 || host2.length() == 0 ||
								host1.equals(host2)){
							return (url1.toString().compareToIgnoreCase(url2.toString()));
						} else {
							String[] tok1 = rotateArray(StringHelper.split(host1, "."));
							String[] tok2 = rotateArray(StringHelper.split(host2, "."));
							int ind1 = tok1.length-1;
							int ind2 = tok2.length-1;
							if (tok1.length >1 && tok1[0].length() == 2 &&
								(tok1[ind1].equals("co") || tok1[ind1].equals("com") ||
								tok1[ind1].equals("net") || tok1[ind1].equals("org") ||
								tok1[ind1].equals("ltd"))){
								rotateArray(tok1);
							}
							if (tok2.length >1 && tok2[0].length() == 2 &&
								(tok2[ind2].equals("co") || tok2[ind2].equals("com") ||
								tok2[ind2].equals("net") || tok2[ind2].equals("org") ||
								tok2[ind2].equals("ltd"))){
								rotateArray(tok2);
							}
							while (ind1 >= 0 && ind2 >= 0){
								int comp = tok1[ind1].compareToIgnoreCase(tok2[ind2]);
								if (comp != 0) return comp;
								ind1--;
								ind2--;
							}
							if (ind1== 0){
								return 1;
							} else {
								return -1;
							}
						}
					} catch (MalformedURLException mue){
						return ((DmozListing)o1).url.url.toString().compareToIgnoreCase(((DmozListing)o2).url.url.toString());
					}
				}
			};
		}
		return urlComparator;
	}

	private static String[] rotateArray(String[] s){
		int ind = s.length-1;
		if (ind < 0) return s;
		String temp = s[ind];
		while (ind > 0){
			s[ind] = s[ind-1];
			ind--;
		}
		s[0] = temp;
		return s;
	}

	private static Comparator titleComparator;
	public static Comparator getTitleComparator(){
		if (titleComparator == null){
			titleComparator =  new Comparator(){
				public int compare(Object o1, Object o2){
				   return ((DmozListing)o1).title.toString().compareToIgnoreCase(((DmozListing)o2).title.toString());
				}
			};
		}
		return titleComparator;
	}

	private static Comparator descriptionComparator;
	public static Comparator getDescriptionComparator(){
		if (descriptionComparator == null) {
			descriptionComparator = new Comparator(){
				public int compare(Object o1, Object o2){
					return ((DmozListing)o1).description.toString().compareToIgnoreCase(((DmozListing)o2).description.toString());
				}
			};
		}
		return descriptionComparator;
	}

	private static Comparator statusComparator;
	public static Comparator getStatusComparator(){
		getURLComparator();
		if (statusComparator == null){
			statusComparator =  new Comparator(){
				public int compare(Object o1, Object o2){
					DmozListing dl1 = (DmozListing)o1;
					DmozListing dl2 = (DmozListing)o2;
					if (dl1.url.status == dl2.url.status){
						boolean ac1 = dl1.url.canAutoCorrect();
						boolean ac2 = dl2.url.canAutoCorrect();
						if (ac1 && !ac2){
							return -1;
						} else if(!ac1 && ac2){
							return 1;
						} else {
							return urlComparator.compare(dl1,dl2);
						}
					} else {
						return dl1.url.status - dl2.url.status;
					}
				}
			};
		}
		return statusComparator;
	}
}
