/*
 * TulipChain is the dmoz browser.  http://ostermiller.org/tulipchain/
 * Copyright (C) 2001 Stephen Ostermiller
 * http://ostermiller.org/contact.pl?regarding=TulipChain
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * See COPYING.TXT for details.
 */

package com.Ostermiller.TulipChain;

import com.Ostermiller.util.Browser;
import java.io.*;
import java.net.*;
import java.util.Properties;
import java.util.Date;
import java.text.MessageFormat;
import java.text.SimpleDateFormat;
import javax.swing.*;

public class ProfileLoader implements Runnable, Haltable {

	private boolean halt = false;

	public void halt(){
		halt = true;
		getThread().interrupt();
	}

	public ProfileLoader(){
	}

	public void run(){
		halt = false;
		loadProfile();
	}

	private void loadProfile(){
		try {
			StringBuffer editorCategoriesBuffer = new StringBuffer();
            StringBuffer greenbustCategoriesBuffer = new StringBuffer();
			URL url;
			HttpURLConnection connection;
			String editorName =  TulipChain.props.getProperty("editor_name");
			String editorPass =  TulipChain.props.getProperty("editor_pass");
			if (editorName == null) return;
			if (editorPass == null){
				url = new URL("http://" + TulipChain.getPublicHost() + "/profiles/" + editorName + ".html");
				connection = (HttpURLConnection) url.openConnection();
			} else {
				url = new URL("http://" + TulipChain.getEditingHost() + "/editors/");
				connection = (HttpURLConnection)url.openConnection();
				TulipChain.setDmozAuthentication(connection, editorName, editorPass);
			}
			TulipChain.setAgent(connection);
			connection.connect();
			InputStream in = connection.getInputStream();
			DmozProfileLexer lex = new DmozProfileLexer(in);
			String t;
			while ((t = lex.next_token()) != null) {
            	if (t.endsWith("&greenbust=1")){
                	t = t.substring(0, t.length()-12);
                    greenbustCategoriesBuffer.append(t + ":");
                } else {
					editorCategoriesBuffer.append(t + ":");
                }
			}
			TulipChain.props.setProperty("editor_categories", editorCategoriesBuffer.toString());
			TulipChain.props.setProperty("greenbust_categories", greenbustCategoriesBuffer.toString());
			TulipChain.tulipChain.editorMenus();
		} catch (IOException ioe){
			JOptionPane.showMessageDialog(
				null,
				ioe.getMessage(),
				TulipChain.labels.getString("load_profile_problem_title"),
				JOptionPane.ERROR_MESSAGE
			);
		}
	}

	private Thread profileLoaderThread = null;
	public void start() {
		if (profileLoaderThread == null || !profileLoaderThread.isAlive()) {
			profileLoaderThread = new Thread(this, "checkUpdate");
			profileLoaderThread.start();
		}
	}

	public Thread getThread(){
		return profileLoaderThread;
	}
}
