/*
 * RedirectLexer.java is a generated file.  You probably want to
 * edit RedirectLexer.lex to make changes.
 */

/*
 * TulipChain is the dmoz browser.	http://ostermiller.org/tulipchain/
 * Copyright (C) 2001-2002 Stephen Ostermiller
 * http://ostermiller.org/contact.pl?regarding=TulipChain
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * See COPYING.TXT for details.
 */

package com.Ostermiller.TulipChain;

import java.io.*;
import com.Ostermiller.util.StringHelper;

%%

%public
%class RedirectLexer
%function next_token
%type String
%caseless

%{
    

    private static final int maxDownload = 0x40000; // 1/4 MB;
    public static final int REDIRECT_TYPE_NONE = 0;
    public static final int REDIRECT_TYPE_META = 1;
    public static final int REDIRECT_TYPE_FRAME = 2;

    /**
     * Prints out tokens from a file or System.in.
     * If no arguments are given, System.in will be used for input.
     * If more arguments are given, the first argument will be used as
     * the name of the file to use as input
     *
     * @param args program arguments, of which the first is a filename
     */
    public static void main(String[] args) {
        InputStream in;
        try {
            if (args.length > 0){
                File f = new File(args[0]);
                if (f.exists()){
                    if (f.canRead()){
                        in = new FileInputStream(f);
                    } else {
                        throw new IOException("Could not open " + args[0]);
                    }
                } else {
                    throw new IOException("Could not find " + args[0]);
                }
            } else {
                in = System.in;
            }
            RedirectLexer shredder = new RedirectLexer(in);
            String t;
            while ((t = shredder.next_token()) != null) {
                System.out.println(t);
            }
        } catch (IOException e){
            System.out.println(e.getMessage());
        }
    }
    
    /** 
     * Meta redirects less than or equal to this many seconds
     * will be reported.
     */
    public void setRedirectTime(int time){
        this.redirectTime = time;        
    }
    
    int redirectTime = 30;
    long count = 0;
    String redirectURL = null;
    boolean redirect = false;
    
    int framesetCount = 0;
    int frameCount = 0;
    String frameRedirect = null;
    int redirectType = REDIRECT_TYPE_NONE;
    int fullFrameNum = -1;
    
    public int getRedirectType(){
        return redirectType;
    }
%}

%unicode
%state DONE
%state META
%state CONTENT
%state CONTENTA
%state ATTRIBUTE
%state FRAME
%state FRAMESET
%state FRAMEDEF
%state FRAMESRC
%state FRAMEVALUE
%state FRAMEELEMENTS
%state FRAMESETDEF
%state FRAMESETVALUE

Digit=([0-9])
Letter=([a-zA-Z])
BLANK=([ ])
TAB=([\t])
FF=([\f])
CR=([\r])
LF=([\n])
EOL=({CR}|{LF}|{CR}{LF})
WhiteSpace=({BLANK}|{TAB}|{FF}|{EOL})
AnyChar=([^A]|[A])

StringLiteral=(([\"][^\"]*[\"])|([\'][^\']*[\']))
NameToken=(({Letter}|{Digit}|[\.\-])+)
Value=({NameToken}|{StringLiteral})
Comment=("<!--"([^\-]|([\-][^\-])|([\-][\-][^\>]))*"-->")

Attributes=({WhiteSpace}*{NameToken}{WhiteSpace}*"="{WhiteSpace}*{Value}*)
RefreshValue=(("refresh"{WhiteSpace}+)|"\"refresh\""|"'refresh'")
RefreshAttribute=(("http-equiv"{WhiteSpace}*"="{WhiteSpace}*{RefreshValue}))


%%

<YYINITIAL> ({Comment}) {
}

<YYINITIAL> ("<"{WhiteSpace}*"meta") {
    yybegin(META);
    redirectURL = null;
    redirect = false;
}

<YYINITIAL> ("<"{WhiteSpace}*"frameset") {
    //System.out.println("YYINITIAL \"<\"{WhiteSpace}*\"frameset\" " + yytext());
    fullFrameNum = -1;
    frameCount = 0;
    framesetCount++;
    yybegin(FRAMESETDEF);
}

<FRAMESETDEF> (("rows"|"cols"){WhiteSpace}*"="{WhiteSpace}*) {
    yybegin(FRAMEELEMENTS);
}

<FRAMEELEMENTS> ({Value}) {
    //System.outprintln("FRAMEELEMENTS {Value} " + yytext());
    String s = yytext();
    if (s.charAt(0) == '\'' || s.charAt(0) == '\"'){
        s = s.substring(1, s.length() - 1);
    }
    String[] values = StringHelper.split(s,",");
    for (int i=0; i<values.length; i++){
        if (values[i].equals("100%")){
            fullFrameNum = i + 1;
        }        
    }
    //System.outprintln("Full Frame Number: " + fullFrameNum);
    yybegin(FRAMESETDEF);
}

<FRAMESETDEF> ({NameToken}{WhiteSpace}*"="{WhiteSpace}*) {
    yybegin(FRAMESETVALUE);
}

<FRAMESETVALUE> ({Value}|{AnyChar}) {
    yybegin(FRAMESETDEF);
}

<FRAMESETDEF> (">") {
    //System.outprintln("FRAMESETDEF \">\" " + yytext());
    yybegin(FRAMESET);
}

<FRAMESET> ("<"{WhiteSpace}*"frameset"({Attributes})*{WhiteSpace}*">") {
    //System.outprintln("FRAMESET \"<\"{WhiteSpace}*\"frameset\"({Attributes})*{WhiteSpace}*\">\" " + yytext());
    if (framesetCount == 1) {
        frameCount++;
    }
    framesetCount++;
}

<FRAMESET> ("<"{WhiteSpace}*"frame") {
    //System.outprintln("FRAMESET \"<\"{WhiteSpace}*\"frame\" " + yytext());
    //System.outprintln("Frame Set Count: " + framesetCount);
    if (framesetCount == 1) {
        yybegin(FRAME);
        frameCount++;
    }
}

<FRAME> ("src"{WhiteSpace}*"="{WhiteSpace}*) {
    //System.outprintln("FRAME \"src\"{WhiteSpace}*\"=\"{WhiteSpace}* " + yytext());
    yybegin(FRAMESRC);
}

<FRAMESRC> ({Value}) {
    //System.outprintln("FRAMESRC {Value} " + yytext());
    String s = yytext();
    if (s.charAt(0) == '\'' || s.charAt(0) == '\"'){
        s = s.substring(1, s.length() - 1);
    } 
	if (s.trim().equals("")){
		// This frame didn't count, there is nothing here.
	   	frameCount--;
	} else {
		frameRedirect = s.trim();
	}     
    yybegin(FRAMESET);   
    //System.outprintln("Frame Count: " + frameCount); 
    if (fullFrameNum == frameCount){
        redirectType = REDIRECT_TYPE_FRAME;
        yybegin(DONE);
        return frameRedirect;
    }
}

<FRAMESRC> ({AnyChar}) {
    yybegin(FRAMESET);
}

<FRAME> ({NameToken}{WhiteSpace}*"="{WhiteSpace}*) {
    //System.outprintln("FRAME {NameToken}{WhiteSpace}*\"=\"{WhiteSpace}* " + yytext());
    yybegin(FRAMEVALUE);
}

<FRAMEVALUE> ({Value}|{AnyChar}) {
    //System.outprintln("FRAMEVALUE {Value}|{AnyChar} " + yytext());
    yybegin(FRAME);
}

<FRAME> (">") {
    //System.outprintln("FRAME \">\" " + yytext());
    yybegin(FRAMESET);
}

<FRAMESET> ("<"{WhiteSpace}*[\/]{WhiteSpace}*"frameset"{WhiteSpace}*">") {
    //System.outprintln("FRAMESET \"<\"{WhiteSpace}*[\\/]{WhiteSpace}*\"frameset\"{WhiteSpace}*\">\" " + yytext());
    framesetCount--;    
    if (framesetCount == 0){
        yybegin(YYINITIAL);
        if (fullFrameNum == -1 && frameCount == 1 && frameRedirect != null){
            redirectType = REDIRECT_TYPE_FRAME;
            yybegin(DONE);
            return frameRedirect;
        }
    }
}

<META, ATTRIBUTE, CONTENT> ({WhiteSpace}*) {
}

<META, ATTRIBUTE, CONTENT> "content" {
    yybegin(CONTENT);
}

<META> {NameToken} {
    yybegin(ATTRIBUTE);
}

<META, ATTRIBUTE, CONTENT> {RefreshAttribute} {
    redirect = true;
}

<ATTRIBUTE> ({WhiteSpace}*"="{WhiteSpace}*{Value}) {
    yybegin(META);
}

<META, ATTRIBUTE, CONTENT> ">" {
    yybegin(YYINITIAL);
    if (redirect && redirectURL != null){
        yybegin(DONE);
        redirectType = REDIRECT_TYPE_META;
        return redirectURL;
    }
}

<CONTENT> ({WhiteSpace}*"="{WhiteSpace}*) {
    yybegin(CONTENTA);
}

<CONTENT> {NameToken} {
    yybegin(META);
}

<CONTENTA> ({WhiteSpace}*) {
}

<CONTENTA> {StringLiteral} {
    yybegin(META);
    String s = yytext();
    StringBuffer timeBuffer = new StringBuffer();
    boolean done = false;
    for(int j=1; !done && j<s.length(); j++){
        char c = s.charAt(j);
        if (c>='0' && c<='9'){
            timeBuffer.append(c);
        } else {
            done = true;
        }
    }
    int time = 0;
    if (timeBuffer.length() > 0 && timeBuffer.length() < 7){
        time = Integer.parseInt(timeBuffer.toString());
    }
    int i = s.indexOf("=");
    if (time <= redirectTime && i!=-1 && (i+1 < s.length()-1)){
    	redirectURL = s.substring(i+1, s.length()-1);
    }
}

<CONTENTA> ({AnyChar}) {
    yybegin(YYINITIAL);
}

<DONE> {AnyChar} {
    return null;
}

<META, ATTRIBUTE, CONTENT> {AnyChar} {
    yybegin(YYINITIAL);
}

<YYINITIAL,FRAMESET,META,CONTENT,CONTENTA,ATTRIBUTE,FRAME,FRAMEDEF,FRAMESRC,FRAMEVALUE,FRAMEELEMENTS,FRAMESETDEF,FRAMESETVALUE> ({AnyChar}) {
    count++;
    if (count >= maxDownload){
        yybegin(DONE);
        return null;
    }
}

