/*
 * TulipChain is the dmoz browser.    http://ostermiller.org/tulipchain/
 * Copyright (C) 2002-2003 Stephen Ostermiller
 * http://ostermiller.org/contact.pl?regarding=TulipChain
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * See COPYING.TXT for details.
 */

package com.Ostermiller.TulipChain;

import java.io.*;
import java.util.*;
import java.net.*;
import com.Ostermiller.util.*;
import java.text.*;

public class TRRSort {

	private final static SimpleDateFormat mediaDateSubmitFormat = new SimpleDateFormat("yyyy.MM.dd");

	public static void main(String[] args) throws IOException {
		String category = null;
		String level = null;
		for (int i=0; i<args.length; i++){
			if (args[i].equals("--level")){
				if(i+1 < args.length){
					level = args[i+1];
					i++;
				} else {
					System.err.println("--level must have an argument.");
					System.exit(1);
				}
			} else if (args[i].startsWith("-")){
				System.err.println("Unknown argument: " + args[i]);
				System.exit(1);
			} else if (category != null){
				System.err.println("Sort one category per run, please.");
				System.exit(1);
			} else {
				category = args[i];
			}
		}
		if (category == null){
			category = "Test/Robozilla/Reds";
		}
		sort(category, level);
	}

	public static void sort(String category, String level) throws IOException {
		if (category.toLowerCase().startsWith("http://dmoz.org/")){
			category = category.substring(16);
		}
		if (category.toLowerCase().startsWith("editors")){
			int start = category.toLowerCase().indexOf("cat=");
			if (start == -1) throw new IOException("Could not parse category string.");
			start += 4;
			int end = category.toLowerCase().indexOf("&", start);
			if (end == -1) end = category.length();
			if (start >= end) throw new IOException("Could not parse category string.");
			category = category.substring(start, end);
		}
		if (category.startsWith("/")){
			category = category.substring(1);
		}
		if (category.endsWith("/")){
			category = category.substring(0, category.length()-1);
		}
		if (!category.startsWith("Test/Kids_and_Teens/Robo_Reds") &&
			!category.startsWith("Test/Robozilla/Reds")){
			throw new IOException("Can only sort Test/Robozilla/Reds categories");
		}
		Vector listings = new Vector();
		DmozCat trrUnrev;
		Properties props = new Properties();
		URL trrpropsurl = ClassLoader.getSystemResource("com/Ostermiller/TulipChain/TRRSort.ini");
		if (trrpropsurl != null) {
			props.load(trrpropsurl.openStream());
		}
		File propsFile = new File(System.getProperty("user.home"), ".java");
		propsFile = new File(propsFile, "TulipChain.ini");
		if (!propsFile.exists()){
			throw new IOException("Editor name and password could not be determined.  Use TulipChain to log into dmoz before running this program.");
		}
		FileInputStream fis = new FileInputStream(propsFile);
		props.load(fis);
		fis.close();
		if (!props.containsKey("editor_name") || !props.containsKey("editor_pass")){
			throw new IOException("Editor name and password could not be determined.  Use TulipChain to log into dmoz before running this program.");
		}
		String editingPort =  props.getProperty("editing.port");
		if (editingPort.equals("80")){
			editingPort = "";
		} else {
			editingPort = ":" + editingPort;
		}

		Vector pairs = new Vector();
		java.util.StringTokenizer cattok = new java.util.StringTokenizer(props.getProperty("TRRCategories"), "\n", false);
		while (cattok.hasMoreTokens()){
			String pair = cattok.nextToken();
			int colon = pair.indexOf(":");
			if (colon > -1 && colon < pair.length() - 1){
				Pair p = new Pair();
				p.from = pair.substring(0,colon);
				p.destination = pair.substring(colon+1);
				if (p.from.length() > 0 &&
						(p.destination.startsWith("Test/Kids_and_Teens/Robo_Reds") ||
						 p.destination.startsWith("Test/Robozilla/Reds"))){
					pairs.add(p);
				}
			}
		}
		URL caturl = new URL("http://dmoz.org" + editingPort + "/editors/editcat-unrev.cgi?cat=" + URLEncoder.encode(category + ((level!=null)?("/" + level):""), "UTF-8"));
		HttpURLConnection caturlConnection = (HttpURLConnection)caturl.openConnection();
		TulipChain.setAgent(caturlConnection);
		TulipChain.setDmozAuthentication(caturlConnection);
		DmozCatLexer catLex = new DmozCatLexer(caturlConnection.getInputStream());
		DmozEntry tok = null;
		for(int i=0; (tok = catLex.next_token()) != null; i++){
			if (tok instanceof DmozListing){
				listings.add(tok);
			}
			if ((i)%100==0){
				System.out.print("+");
			}
			if ((i+1)%5000==0){
				System.out.println();
			}
		}
		System.out.println();
		Vector moveNotes = new Vector();
		for(int i=0; i<listings.size(); i++){
			moveNotes.clear();
			try {
				DmozListing listing = (DmozListing)listings.elementAt(i);
				String editurlString = (
					"http://dmoz.org" + editingPort + "/editors/editunrev.cgi" + "?" +
					"site=" + listing.getIDNum() + "&" +
					"cat=" + URLEncoder.encode(category + ((level!=null)?("/" + level):""), "UTF-8")
				);
				URL editurl = new URL(editurlString);
				HttpURLConnection editurlConnection = (HttpURLConnection)editurl.openConnection();
				TulipChain.setAgent(editurlConnection);
				TulipChain.setDmozAuthentication(editurlConnection);
				TRRLexer lex = new TRRLexer(editurlConnection.getInputStream());
				DmozMoveNote note;
				while ((note=lex.next_token())!=null){
					moveNotes.add(note);
				}
				String toSearch = category + ((level!=null)?("/" + level):"");
				String topcat = null;
				for(int j=moveNotes.size()-1; topcat==null && j>=0; j--){
					note = (DmozMoveNote)moveNotes.elementAt(j);
					for (int k=0; topcat==null && k<pairs.size(); k++){
						if (note.from.startsWith(((Pair)pairs.elementAt(k)).from)){
							topcat = ((Pair)pairs.elementAt(k)).destination;
						}
					}
					if (topcat == null){
						toSearch = note.from;
					}
				}
				if (topcat == null) {
					StringBuffer sb = new StringBuffer();
					for (int j=0; j<moveNotes.size(); j++){
						sb.append((DmozMoveNote)moveNotes.elementAt(j));
					}
					throw new IOException("No from category found for " + listing.getURL() + " in " + sb.toString());
				}
				String moveurlString = (
					"http://dmoz.org" + editingPort + "/editors/editunrev2.cgi" + "?" +
					"site=" + listing.getIDNum() + "&" +
					"cat=" + URLEncoder.encode(category + ((level!=null)?("/" + level):""), "UTF-8") + "&" +
					"typecat=" + URLEncoder.encode(topcat + ((level!=null)?("/" + level):""), "UTF-8") + "&" +
					"url=" + URLEncoder.encode(listing.getURL(), "UTF-8") + "&" +
					"newurl=" + URLEncoder.encode(listing.getURL(), "UTF-8") + "&" +
					"newtitle=" + URLEncoder.encode(listing.getTitle(), "UTF-8") + "&" +
					"newdesc=" + URLEncoder.encode(listing.getDescription(), "UTF-8") + "&" +
					"newnote=" + URLEncoder.encode("Move by automatic TRR sorting tool", "UTF-8") + "&" +
					((listing.getMediaDate() != null)?("date=" + URLEncoder.encode(mediaDateSubmitFormat.format(listing.getMediaDate()), "UTF-8") + "&"):"") +
					"omit=unrev" + "&" +
					"submit=Update"

				);
				URL moveurl = new URL(moveurlString);
				HttpURLConnection moveurlConnection = (HttpURLConnection)moveurl.openConnection();
				TulipChain.setAgent(moveurlConnection);
				TulipChain.setDmozAuthentication(moveurlConnection);
				InputStream in = moveurlConnection.getInputStream();
				while (in.read() != -1);
				System.out.println(topcat + ((level!=null)?("/" + level):"") + " " + listing.getURL());

			} catch (IOException x){
				System.err.println(x);
			}
		}
	}

	private static class Pair {
		String from;
		String destination;
	}
}

