/*
 * TulipChain is the dmoz browser.    http://ostermiller.org/tulipchain/
 * Copyright (C) 2001-2003 Stephen Ostermiller
 * http://ostermiller.org/contact.pl?regarding=TulipChain
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * See COPYING.TXT for details.
 */

package com.Ostermiller.TulipChain;

import com.Ostermiller.util.Browser;
import com.Ostermiller.util.PasswordDialog;
import com.Ostermiller.util.Base64;
import com.Ostermiller.util.StringHelper;
import javax.swing.*;
import javax.swing.table.*;
import javax.swing.event.*;
import javax.swing.tree.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.net.*;
import java.util.*;
import java.text.*;

public class TulipChain extends JFrame {

	public static String version = "6.01";

	/**
	 * Locale specific strings displayed to the user.
	 */
	static ResourceBundle labels = ResourceBundle.getBundle("com.Ostermiller.TulipChain.TulipChain",  Locale.getDefault());

	static ResourceBundle reportLabels;
    
	final static SimpleDateFormat mediaDateSubmitFormat = new SimpleDateFormat("yyyy.MM.dd");
	private final static SimpleDateFormat javascriptDateFormat = new SimpleDateFormat("MMMM dd, yyyy");
	private Date fourDays;

	private static final String[] topCats = {
		"Adult/",
		"Arts/",
		"Bookmarks/",
		"Business/",
		"Computers/",
		"Games/",
		"Health/",
		"Home/",
		"Kids_and_Teens/",
		"Netscape/",
		"News/",
		"Recreation/",
		"Reference/",
		"Regional/",
		"Science/",
		"Shopping/",
		"Society/",
		"Sports/",
		"Test/",
		"World/",
	};

	static Properties props = new Properties();
	static final ThreadManager threadManager = new ThreadManager();
	static DmozTableModel tableModel;
	static DmozCat topCat = new DmozCat(topCats);
	static DmozCat lastCatSelect;
	static JTree tree;
	static JTable table;

	static String getEditingPort(){
		String s = props.getProperty("editing.port");
		if (s == null) return "";
		if (s.equals("80")) return "";
		return (":" + s);
	}

    static String getEditingHost(){
		return props.getProperty("editing.host", "editors.dmoz.org:8080");
	}

    static String getPublicHost(){
		return props.getProperty("public.host", "dmoz.org");
	}

    static String getCanonicalHost(){
		return props.getProperty("canonical.host", "dmoz.org");
	}

    static boolean useEditingAsPublicWhenPossible(){
    	return props.getProperty("public.pages.from.editing.host", "true").equals("true");
    }

	public static synchronized void setStatus(String status){
		TulipChain.status.setText(status);
	}

	private JSplitPane splitPane;
	private static DefaultTreeModel treeModel;
	private static JLabel status = new JLabel(TulipChain.labels.getString("welcome_message"));

	private JPopupMenu treePopup;
	private JPopupMenu tablePopup;

	private int lastRowSelect;

	private JTextField urlBar = new JTextField();
	private JButton stopButton;

	private void setProxy(){
		if (props != null && !props.getProperty("proxy.useSystem", "true").equals("true")){
			if (props.containsKey("proxy.host")){
				Properties prop = System.getProperties();
				prop.put("http.proxyHost", props.getProperty("proxy.host"));
				prop.put("http.proxyPort", props.getProperty("proxy.port", "80"));
				prop.put("proxySet", "true");
				prop.put("proxyHost", props.getProperty("proxy.host"));
				prop.put("proxyPort", props.getProperty("proxy.port", "80"));
			} else {
				Properties prop = System.getProperties();
				prop.remove("http.proxyHost");
				prop.remove("http.proxyPort");
				prop.put("proxySet", "false");
				prop.remove("proxyHost");
				prop.remove("proxyPort");
			}
		}
	}

    static String getDmozAuthentication(){
    	if (props.getProperty("editor_name") == null || props.getProperty("editor_pass") == null) return null;
		return (
			"Basic " +
			Base64.encode(
				props.getProperty("editor_name") +
				":" +
				props.getProperty("editor_pass")
			)
		);
	}

	static void setDmozAuthentication(HttpURLConnection connection){
		setDmozAuthentication(
			connection,
			props.getProperty("editor_name"),
			props.getProperty("editor_pass")
		);
	}

   static void setDmozAuthentication(HttpURLConnection connection, String name, String pass){
		connection.setRequestProperty(
			"Authorization",
			"Basic " +
			Base64.encode(
				name +
				":" +
				pass
			)
		);
	}

    static String getAgent(){
    	return (
        	"TulipChain/" + version + " " +
			"(http://ostermiller.org/tulipchain/)" + " " +
			"Java/" + System.getProperty("java.version") + " " +
			"(" + System.getProperty("java.vendor.url") + ")" + " " +
			System.getProperty("os.name").replace(' ', '_') + "/" + System.getProperty("os.version")
        );
    }

    static String getProxyAUthorization(){
    	if (props != null && !props.getProperty("proxy.useSystem", "true").equals("true")){
			if (props.containsKey("proxy.host")){
				if (props.containsKey("proxy.user")){
                	return (
                    	"Basic " +  Base64.encode(
							props.getProperty("proxy.user") + ":" + props.getProperty("proxy.pass", "")
						)
                    );
				}
			}
		}
        return null;
	}

	static void setAgent(HttpURLConnection connection){
		connection.setRequestProperty(
			"User-Agent",
			getAgent()
		);

        String proxyAuthorization = getProxyAUthorization();
        if (proxyAuthorization != null){
            connection.setRequestProperty(
            	"Proxy-Authorization",
                proxyAuthorization
            );
        }
	}

	/**
	 * Editor features menu item
	 */
	private JCheckBoxMenuItem editorFeaturesMenuItem;

	public static DefaultTreeModel getTreeModel(){
		return treeModel;
	}

	private JScrollPane treeView;
	private Locale locale;

	public TulipChain() {
		super();

		setIconImage(
			new ImageIcon(
				ClassLoader.getSystemResource(
					"com/Ostermiller/TulipChain/tulipchain.png"
				)
			).getImage()
		);

		loadProperties();
		if (!props.getProperty("locale","lang_default").equals("lang_default")){
			String[] locElem = StringHelper.split(props.getProperty("locale"),"_");
			if (locElem.length >= 3) locale = new Locale(locElem[0], locElem[1], locElem[2]);
			if (locElem.length == 2) locale = new Locale(locElem[0], locElem[1]);
			if (locElem.length == 1) locale = new Locale(locElem[0]);
			labels = ResourceBundle.getBundle(
				"com.Ostermiller.TulipChain.TulipChain",
				locale
			);
			Browser.setLocale(locale);
		}
		UIManager.put("OptionPane.yesButtonText", labels.getString("dialog.yes"));
		UIManager.put("OptionPane.noButtonText", labels.getString("dialog.no"));
		UIManager.put("OptionPane.cancelButtonText", labels.getString("dialog.cancel"));
		UIManager.put("OptionPane.okButtonText", labels.getString("dialog.ok"));

		setTitle(TulipChain.labels.getString("window_title"));
		threadManager.setManager(this);
		topCat.setTulip(this);

		threadManager.start();

		JScrollPane tableView = initTable();

		treeView = initTree();

		addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				exitRoutine();
			}
		});

		stopButton = new JButton(TulipChain.labels.getString("stop_button"));
		stopButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e){
				threadManager.halt();
			}
		});

		urlBar.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e){
				threadManager.selectCat(urlBar.getText());
			}
		});

		JPanel topPanel = new JPanel(new BorderLayout());
		topPanel.add(stopButton, BorderLayout.WEST);
		topPanel.add(urlBar, BorderLayout.CENTER);

		buildMenus();

		Calendar cal = Calendar.getInstance();
		cal.add(Calendar.DATE, 4);
		fourDays= cal.getTime();

		splitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, false, treeView, tableView);
		getContentPane().add(splitPane, BorderLayout.CENTER);
		getContentPane().add(status, BorderLayout.SOUTH);
		getContentPane().add(topPanel, BorderLayout.NORTH);

		try {
			ThreadManager.MAX_THREADS = Integer.parseInt(props.getProperty("max_threads", "" + ThreadManager.MAX_THREADS), 10);
		} catch (NumberFormatException x){
		}
		int x, y;
		try {
			x = Integer.parseInt(props.getProperty("window_size_x", "600"), 10);
			y = Integer.parseInt(props.getProperty("window_size_y", "440"), 10);
		} catch (NumberFormatException e){
			x = 600;
			y = 440;
		}
		setSize(x,y);
		try {
			x = Integer.parseInt(props.getProperty("window_location_x", "50"), 10);
			y = Integer.parseInt(props.getProperty("window_location_y", "50"), 10);
		} catch (NumberFormatException e){
			x = 50;
			y = 50;
		}
		setLocation(x,y);
		int splitpaneLocation = 100;
		try {
			splitpaneLocation = Integer.parseInt(props.getProperty("split_pane_divider", ""), 10);
			if (splitpaneLocation <= 0){
				splitpaneLocation = 100;
			}
		} catch (NumberFormatException e){
		}
		splitPane.setDividerLocation(splitpaneLocation);
		try {
			int w = Integer.parseInt(props.getProperty("tree_pane_width", ""), 10);
			Dimension d = treeView.getPreferredSize();
			if (w > 0){
				d.width = w;
				//treeView.setSize(d);
				treeView.setPreferredSize(d);
			}
		} catch (NumberFormatException e){
		}
		for (int i = 0; i < table.getColumnCount(); i++) {
			int width;
			try {
			   width = Integer.parseInt(props.getProperty("columnWidth" + i, "100"), 10);
			} catch (NumberFormatException e){
				width = 100;
			}
			table.getColumnModel().getColumn(i).setPreferredWidth(width);
		}
        
        String startLoad = props.getProperty("start_load", "last");
        if (startLoad.equals("last")){
        	threadManager.selectCat(props.getProperty("last_cat", "/"));
        } else if (!startLoad.equals("none")){
        	threadManager.selectCat(props.getProperty("start_load_cat", "/"));
        }
        		
		if(props.getProperty("latest_check", "true").equals("true")){
			Date last = UpdateCheck.dateFormat.parse(
				props.getProperty("latest_check_time", "1900 01 01 01 01"),
				new ParsePosition(0)
			);
			Calendar compareTime = Calendar.getInstance();
			int hours;
			try {
				hours = Integer.parseInt(props.getProperty("latest_hours", "24"), 10);
			} catch (NumberFormatException nfe){
				hours = 24;
				props.put("latest_hours", "24");
			}
			compareTime.add(Calendar.HOUR, -hours);
			if (last.before(compareTime.getTime())){
				threadManager.checkForUpdates(false, false);
			}
		}

		updateRedirectErrors(
			props,
			"redirect.errors",
			props.getProperty("redirect.errors.old", ""),
			props.getProperty("redirect.errors", ""),
			props.getProperty("redirect.errors.new", "")
		);
		updateRedirectErrors(
			props,
			"redirect.oks",
			props.getProperty("redirect.oks.old", ""),
			props.getProperty("redirect.oks", ""),
			props.getProperty("redirect.oks.new", "")
		);
		setProxy();

		setVisible(true);

		editorMenus();

		initPopups();
	}

	static void updateRedirectErrors(Properties props, String property,
			String oldValues, String currentValues, String suggestedValues){
		StringTokenizer tok;
		HashSet current = new HashSet();
		HashSet old = new HashSet();
		Iterator it;
		StringBuffer value;

		tok = new StringTokenizer(currentValues, ";\n", false);
		while (tok.hasMoreTokens()){
			String token = tok.nextToken().trim();
			if (token.length() > 0) current.add(token);
		}

		tok = new StringTokenizer(oldValues, ";\n", false);
		while (tok.hasMoreTokens()){
			String token = tok.nextToken().trim();
			if (token.length() > 0 && !current.contains(token)) old.add(token);
		}

		tok = new StringTokenizer(suggestedValues, ";\n", false);
		while (tok.hasMoreTokens()){
			String token = tok.nextToken().trim();
			if (token.length() > 0 && !old.contains(token)) current.add(token);
		}

		it = current.iterator();
		value = new StringBuffer();
		while (it.hasNext()){
			value.append(it.next()).append(";");
		}
		props.put(property, value.toString());

		it = old.iterator();
		value = new StringBuffer();
		while (it.hasNext()){
			value.append(it.next()).append(";");
		}
		props.put(property + ".old", value.toString());

		props.remove(property + ".new");

	}

	static TulipChain tulipChain;
	public static void main(String[] args){
		tulipChain = new TulipChain();
	}

	/**
	 * Load the properties file.
	 */
	private void loadProperties(){
		try{
			URL url = ClassLoader.getSystemResource("com/Ostermiller/TulipChain/TulipChain.ini");
			if (url != null) {
				props.load(url.openStream());
			}
		} catch (IOException e){
			System.err.println(ClassLoader.getSystemResource("com/Ostermiller/TulipChain/TulipChain.ini"));
			System.err.println("com/Ostermiller/TulipChain/TulipChain.ini not found");
		}
		try{
			File propsFile = new File(System.getProperty("user.home"), ".java");
			propsFile = new File(propsFile, "TulipChain");
			propsFile.mkdirs();
			propsFile = new File(propsFile, "TulipChain.ini");
			FileInputStream fis = new FileInputStream(propsFile);
			props.load(fis);
			fis.close();
		} catch (IOException e){
		}
		if (props.containsKey("browser_open")){
			props.setProperty("com.Ostermiller.util.Browser.open", props.getProperty("browser_open"));
			props.remove("browser_open");
		}
		Browser.load(props);
	}

	/**
	 * Store the properties file.
	 */
	private void storeProperties(){
		Browser.save(props);
		try{
			 File propsFile = new File(System.getProperty("user.home"), ".java");
			 propsFile = new File(propsFile, "TulipChain");
			 propsFile.mkdirs();
			 propsFile = new File(propsFile, "TulipChain.ini");
			 FileOutputStream f = new FileOutputStream(propsFile);
			 props.store(f, "User Preferences for TulipChain");
			 f.close();
		 } catch (IOException e){
			 System.err.println("Could not open TulipChain.ini");
		 }
	}

	private Comparator lastComparator = null;

	private JScrollPane initTable(){
		tableModel = new DmozTableModel(topCat);
		table = new JTable(tableModel);
		table.setDefaultRenderer(URLStatus.class, new URLStatusRenderer(true));
		table.setDefaultRenderer(String.class, new TextRenderer());
		table.getTableHeader().addMouseListener(new MouseAdapter(){
			public void mouseClicked(MouseEvent e) {
				int column = table.convertColumnIndexToModel(table.getColumnModel().getColumnIndexAtX(e.getX()));
				DmozCat cat = (DmozCat)tree.getLastSelectedPathComponent();
				Comparator c;
				switch (column){
					case 0:{
						c = DmozListing.getURLComparator();
						if (lastComparator == c){
							c = DmozListing.getStatusComparator();
						}
					} break;
					case 1:{
						c = DmozListing.getTitleComparator();
					} break;
					case 2:{
						c = DmozListing.getDescriptionComparator();
					} break;
					default: {
						c = DmozListing.getURLComparator();
					} break;
				}
				lastComparator = c;
				cat.sort(c);
			}
		});
		table.addKeyListener(new KeyAdapter(){
			public void keyReleased(KeyEvent e){
				if (lastCatSelect.getName().endsWith("/unreviewed /")){
					if (e.getKeyCode() == KeyEvent.VK_DELETE){
						deleteRows(table.getSelectedRows());
					}
				}
			}
		});
		table.addMouseListener(new MouseAdapter() {
			public void mousePressed(MouseEvent e) {
				int selRow = table.rowAtPoint(e.getPoint());
				if (!table.isRowSelected(selRow)){
					// select the row
					table.setRowSelectionInterval(selRow, selRow);
				}
				if(selRow != -1){
					if(e.isPopupTrigger() || (e.getModifiers() & InputEvent.BUTTON3_MASK) != 0) {
						tablePopup.removeAll();
						tablePopup.add(tableOpen);
						if (props.getProperty("editor_features", "false").equals("true")
                        		&& (canEdit(lastCatSelect.getName())
                                || canGreenbust(lastCatSelect.getName()))){
							tablePopup.add(tableEdit);
							boolean autocorrect = true;
							int[] selectedRows = table.getSelectedRows();
							for (int i=0; i<selectedRows.length; i++){
								URLStatus s = (URLStatus)tableModel.getValueAt(selectedRows[i], 0);
								if (!s.canAutoCorrect()) {
									autocorrect = false;
								}
							}
							if (autocorrect){
								tablePopup.add(tableAutoCorrect);
							}
						} else if (!(lastCatSelect.getName().endsWith("/unreviewed /"))){
							tablePopup.add(tableSuggest);
						}
						tablePopup.add(tableCheck);
						if (lastCatSelect.getName().endsWith("/unreviewed /")){
							tablePopup.add(tableDelete);
						}
						showPopUpSmart(tablePopup, e.getComponent(), e.getX(), e.getY());
					}
					if(e.getClickCount() >= 2 && (e.getModifiers() & InputEvent.BUTTON1_MASK) != 0) {
						String url = ((URLStatus)tableModel.getValueAt(selRow, 0)).url;
						try {
							Browser.displayURL(url);
						} catch (IOException ioe){
							System.err.println(ioe.getMessage());
						}
					}
				}
			}
		});
		//Create the scroll pane and add the table to it.
		return new JScrollPane(table, ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED,
			ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);
	}

	private JScrollPane initTree(){
		treeModel = new DefaultTreeModel(topCat);
		tree = new JTree(treeModel);
		tree.addMouseListener(new MouseAdapter() {
			public void mousePressed(MouseEvent e) {
				int selRow = tree.getRowForLocation(e.getX(), e.getY());
				if (!tree.isRowSelected(selRow)){
					// select the row
					tree.setSelectionRow(selRow);
				}
				if(selRow != -1) {
					DmozCat cat = (DmozCat)tree.getPathForLocation(e.getX(), e.getY()).getLastPathComponent();
					if (cat != topCat){
						if(e.isPopupTrigger() || (e.getModifiers() & InputEvent.BUTTON3_MASK) != 0) {
							treePopup.removeAll();
							treePopup.add(treeRefresh);
							if (props.getProperty("editor_features", "false").equals("true") && 
                            		(canEdit(cat.getName()) || 
                                    canGreenbust(cat.getName()))){
								treePopup.add(treeEdit);
							} else if (!cat.getName().endsWith("/unreviewed /")) {
								treePopup.add(treeBecomeEditor);
							}
							treePopup.add(treeCheck);
							if (!cat.getName().endsWith("/unreviewed /")){
								treePopup.add(treeSpell);
								if (cat.spidered()){
									treePopup.add(treeExpandAll);
									treePopup.add(treeCheckAll);
								} else {
									treePopup.add(treeSpider);
								}
							}
							treePopup.add(treeReport);
							showPopUpSmart(treePopup, e.getComponent(), e.getX(), e.getY());
						}
					}
				}
			}
		});

		//Listen for when the selection changes.
		tree.addTreeSelectionListener(new TreeSelectionListener() {
			public void valueChanged(TreeSelectionEvent e) {
				DmozCat node = (DmozCat)tree.getLastSelectedPathComponent();
				if (node == null){
					 return;
				} else {
					if (node != topCat && node.getLastUpdateTime() == null){
						status.setText(
							MessageFormat.format(
								TulipChain.labels.getString("loading_status"),
								new Object[] {
									node.getName()
								}
							)
						);
						threadManager.loadCat(node);
					}
					tableModel.setCat(node);
					lastCatSelect = node;
					urlBar.setText(node.getName());
					resizeRows();
				}
			}
		});
		tree.addTreeWillExpandListener(new TreeWillExpandListener() {
			public void treeWillExpand(TreeExpansionEvent e) throws ExpandVetoException {
				DmozCat node = (DmozCat)(e.getPath().getLastPathComponent());
				if (node != topCat && node.getLastUpdateTime() == null){
					status.setText(
						MessageFormat.format(
							TulipChain.labels.getString("loading_status"),
							new Object[] {
								node.getName()
							}
						)
					);
					threadManager.loadCat(node);
					//resizeRows();
				}
			}

			public void treeWillCollapse(TreeExpansionEvent e) {
			}
		});

		return new JScrollPane(tree);
	}

	private void showPopUpSmart(JPopupMenu popup, Component component, int x, int y){
		popup.show(component, x, y);
		boolean reshow = false;
		Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
		Dimension popupSize = popup.getSize();
		Point screenLocation = popup.getLocationOnScreen();
		x = screenLocation.x;
		y = screenLocation.y;
		if (screenLocation.x + popupSize.width > screenSize.width){
			x = screenSize.width - popupSize.width;
			reshow = true;
		}
		if (screenLocation.y + popupSize.height > screenSize.height){
			y = screenSize.height - popupSize.height;
			reshow = true;
		}
		if (reshow){
			popup.setLocation(x,y);
		}
	}

	static void resizeRows(){
		synchronized (table){
			/*for (int i=0; i<table.getRowCount(); i++){
				int maxHeight = 0;
				for (int j=0; j<table.getColumnCount(); j++){
					TableCellRenderer renderer = table.getCellRenderer(i, j);
					Component component = renderer.getTableCellRendererComponent(
						table, table.getModel().getValueAt(i, j), false, false, i, j
					);
					component.setSize(table.getColumnModel().getColumn(j).getWidth(), 0);
					int height = component.getPreferredSize().height;
					if (height > maxHeight){
						maxHeight = height;
					}
				}
				if (maxHeight != 0 && table.getRowHeight(i) != maxHeight){
					table.setRowHeight(i, maxHeight);
				}
			}*/
		}
	}

	private JMenuItem treeRefresh, treeEdit, treeBecomeEditor, treeCheck,
		treeSpider, treeCheckAll, treeExpandAll, treeReport, treeSpell,
		tableOpen, tableSuggest, tableCheck, tableEdit, tableDelete, tableAutoCorrect;

	private void deleteRows(int[] rows){
		String message = "";
		while (message.trim().equals("")){
			JOptionPane pane = new JOptionPane(
				MessageFormat.format(
					TulipChain.labels.getString("delete_confirm_message"),
					new Object[] {
						new Integer(rows.length)
					}
				),
				JOptionPane.QUESTION_MESSAGE,
				JOptionPane.OK_CANCEL_OPTION
			);
			pane.setWantsInput(true);
			if (locale != null){
				pane.setLocale(locale);
			}
			JDialog dialog = pane.createDialog(
				TulipChain.this,
				TulipChain.labels.getString("delete_confirm")
			);
			if (dialog != null){
				dialog.setLocale(locale);
			}
			dialog.show();
			if (pane.getValue() == null || ((Integer)pane.getValue()).intValue() != JOptionPane.OK_OPTION){
				message = null;
			} else {
				message = (String)pane.getInputValue();
			}
			if (message == null) return;
		}
		DmozListing[] listings = new DmozListing[rows.length];
		for (int i=0; i<rows.length; i++){
			listings[i] = ((URLStatus)tableModel.getValueAt(rows[i], 0)).getListing();
		}
		DmozDeleter toDelete = new DmozDeleter(lastCatSelect, listings, message);
		threadManager.delete(toDelete);
	}

	private void initPopups(){

		ActionListener treePopupListener = new ActionListener() {
			public void actionPerformed(ActionEvent e){
				if (e.getSource() == treeCheck){
					lastCatSelect.checkAll();
				} else if (e.getSource() == treeSpell){
					try {
						String catName = lastCatSelect.getName();
						Browser.displayURL("http://odp.rpfuller.org/spell/?cd=1&cat=" + catName);
					} catch (IOException ioe){
						System.err.println(ioe.getMessage());
					}
				} else if (e.getSource() == treeCheckAll){
					lastCatSelect.checkAll(true);
				} else if (e.getSource() == treeExpandAll){
					expandAll(lastCatSelect);
				} else if (e.getSource() == treeReport){
					try {
						if (reportDialog == null){
							reportDialog = new ReportDialog(TulipChain.this, props);
						} else {
							reportDialog.setProps(props);
						}
						reportDialog.show();
						if (reportDialog.ok()){
							generateReport(lastCatSelect, props);
						}
					} catch (IOException x){
						System.err.println(x.getMessage());
					}
				} else if (e.getSource() == treeBecomeEditor){
					try {
						if (props.getProperty("editor_features", "false").equals("true")){
							Browser.displayURL("http://" + getEditingHost() + "/editors/editcat.cgi?cat=" + lastCatSelect.getName());
						} else {
							Browser.displayURL("http://" + getEditingHost() + "/cgi-bin/apply.cgi?where=" + lastCatSelect.getName());
						}
					} catch (IOException ioe){
						System.err.println(ioe.getMessage());
					}
				} else if (e.getSource() == treeEdit){
					try {
						String cat = lastCatSelect.getName();
						if (cat.endsWith("/unreviewed /")){
							cat = cat.substring(0, cat.length()-12);
							Browser.displayURL("http://" + getEditingHost() + "/editors/editcat-unrev.cgi?cat=" + cat + "&chainsaw=1");
						} else {
							Browser.displayURL("http://" + getEditingHost() + "/editors/editcat.cgi?cat=" + cat);
						}
					} catch (IOException ioe){
						System.err.println(ioe.getMessage());
					}
				} else if (e.getSource() == treeRefresh){
					threadManager.loadCat(lastCatSelect);
				} else if (e.getSource() == treeSpider){
					threadManager.spiderCat(lastCatSelect);
				}
			}
		};

		ActionListener tablePopupListener = new ActionListener() {
			public void actionPerformed(ActionEvent e){
				synchronized (table) {
					int[] selectedRows = table.getSelectedRows();
					if (e.getSource() == tableDelete){
						deleteRows(selectedRows);
						return;
					}
					int mayAutoCorrect = JOptionPane.NO_OPTION;
					if (e.getSource() == tableAutoCorrect){
						if (props.getProperty("autocorrect.noconfirm", "false").equals("true")){
							mayAutoCorrect = JOptionPane.YES_OPTION;
						} else {
							mayAutoCorrect = JOptionPane.showConfirmDialog(
								TulipChain.this,
								TulipChain.labels.getString("autocorrect_confirm_message"),
								TulipChain.labels.getString("autocorrect_confirm_title"),
								JOptionPane.YES_NO_OPTION
							);
						}
					}
					for (int i=0; i<selectedRows.length; i++){
						String cat = lastCatSelect.getName();
						URLStatus urlStatus = (URLStatus)tableModel.getValueAt(selectedRows[i], 0);
						if (e.getSource() == tableSuggest){
							String url = URLEncodeToCatCharset(urlStatus.url,lastCatSelect);
							try {
								Browser.displayURL("http://" + getCanonicalHost() + "/cgi-bin/update2.cgi?lk=&id=&where=" + cat + "&url=" + url);
							} catch (IOException ioe){
								System.err.println(ioe.getMessage());
							}
						} if (e.getSource() == tableAutoCorrect){
							if (mayAutoCorrect == JOptionPane.YES_OPTION){
								threadManager.autoCorrect(new AutoCorrector(urlStatus.getListing()));
							}
						} else if (e.getSource() == tableEdit){
							String url = URLEncodeToCatCharset(urlStatus.url,lastCatSelect);
							try {
								if (cat.endsWith("/unreviewed /")){
									cat = cat.substring(0, cat.length()-12);
									String idNum = urlStatus.getListing().getIDNum();
									Browser.displayURL("http://" + getEditingHost() + "/editors/editunrev.cgi?site=" + idNum + "&cat=" + cat + "&chainsaw=1");
								} else {
									Browser.displayURL("http://" + getEditingHost() + "/editors/editurl.cgi?url=" + url + "&cat=" + cat);
								}
							} catch (IOException ioe){
								System.err.println(ioe.getMessage());
							}
						} else if (e.getSource() == tableOpen){
							try {
								Browser.displayURL(urlStatus.url);
							} catch (IOException ioe){
								System.err.println(ioe.getMessage());
							}
						} else if (e.getSource() == tableCheck){
							threadManager.checkSite(urlStatus);
						}
					}
				}
			}
		};


		tablePopup = new JPopupMenu();
		treePopup = new JPopupMenu();

		treeBecomeEditor = new JMenuItem(TulipChain.labels.getString("become_editor"));
		treeBecomeEditor.addActionListener(treePopupListener);
		treeEdit = new JMenuItem(TulipChain.labels.getString("edit"));
		treeEdit.addActionListener(treePopupListener);
		treeRefresh = new JMenuItem(TulipChain.labels.getString("refresh"));
		treeRefresh.addActionListener(treePopupListener);
		treeSpider = new JMenuItem(TulipChain.labels.getString("spider"));
		treeSpider.addActionListener(treePopupListener);
		treeCheck = new JMenuItem(TulipChain.labels.getString("check_sites"));
		treeCheck.addActionListener(treePopupListener);
		treeSpell = new JMenuItem(TulipChain.labels.getString("spell_check"));
		treeSpell.addActionListener(treePopupListener);
		treeCheckAll = new JMenuItem(TulipChain.labels.getString("check_all_sites"));
		treeCheckAll.addActionListener(treePopupListener);
		treeExpandAll = new JMenuItem(TulipChain.labels.getString("expand_all"));
		treeExpandAll.addActionListener(treePopupListener);
		treeReport = new JMenuItem(TulipChain.labels.getString("generate_Report"));
		treeReport.addActionListener(treePopupListener);

		tableOpen = new JMenuItem(TulipChain.labels.getString("open_listing"));
		tableOpen.addActionListener(tablePopupListener);
		tableSuggest = new JMenuItem(TulipChain.labels.getString("suggest_listing"));
		tableSuggest.addActionListener(tablePopupListener);
		tableAutoCorrect = new JMenuItem(TulipChain.labels.getString("report_auto_correct"));
		tableAutoCorrect.addActionListener(tablePopupListener);
		tableCheck = new JMenuItem(TulipChain.labels.getString("check_listing"));
		tableCheck.addActionListener(tablePopupListener);
		tableEdit = new JMenuItem(TulipChain.labels.getString("edit_listing"));
		tableEdit.addActionListener(tablePopupListener);
		tableDelete = new JMenuItem(TulipChain.labels.getString("delete_listing"));
		tableDelete.addActionListener(tablePopupListener);
	}

	private static String URLEncodeToCatCharset(String s,DmozCat cat){
		try {
			return URLEncoder.encode(s, cat.getEncoding());
		} catch (java.io.UnsupportedEncodingException uee){
			return (MessageFormat.format(
				TulipChain.labels.getString("unsupported_encoding"),
				new Object[] {
					"UTF-8"
				}
			));
		}
	}

	private JMenuBar menuBar;
	private JMenuItem exitItem, aboutItem, websiteItem, forumItem, dmozItem,
		licenseItem, OptionsItem, updateItem;
	private JCheckBoxMenuItem editorFeaturesItem;

	private OptionsDialog optionsDialog;
	private ReportDialog reportDialog;

	private void buildMenus(){

		ActionListener menuListener = new ActionListener(){
			public void actionPerformed(java.awt.event.ActionEvent event){
				Object source = event.getSource();
				if (source == exitItem){
					setVisible(false);
					exitRoutine();
				} else if (source == aboutItem){
					JOptionPane.showMessageDialog(
						TulipChain.this,
						MessageFormat.format(
							TulipChain.labels.getString("about_message"),
							new Object[] {
								version,
								"2001-2002 Stephen Ostermiller http://ostermiller.org/contact.pl?regarding=TulipChain"
							}
						),
						TulipChain.labels.getString("about_title"),
						JOptionPane.INFORMATION_MESSAGE
					);
				} else if (source == websiteItem){
					try {
						Browser.displayURL("http://ostermiller.org/tulipchain/");
					} catch (IOException ioe){
						System.err.println(ioe.getMessage());
					}
				} else if (source == forumItem){
					try {
						Browser.displayURL("http://forums.dmoz.org/forum/viewtopic.php?t=850052");
					} catch (IOException ioe){
						System.err.println(ioe.getMessage());
					}
				} else if (source == dmozItem){
					try {
						Browser.displayURL("http://" + getCanonicalHost() + "/");
					} catch (IOException ioe){
						System.err.println(ioe.getMessage());
					}
				} else if (source == OptionsItem){
					if (optionsDialog == null){
						optionsDialog = new OptionsDialog(TulipChain.this, props);
					}
					optionsDialog.setProps(props);
					optionsDialog.show();
					setProxy();
				} else if (source == updateItem){
					threadManager.checkForUpdates(true, true);
				} else if (source == licenseItem){
					try {
						Browser.displayURL("http://www.gnu.org/copyleft/gpl.html");
					} catch (IOException ioe){
						System.err.println(ioe.getMessage());
					}
				}
			}
		};

		ItemListener itemListener = new ItemListener(){
			public void itemStateChanged(java.awt.event.ItemEvent event){
				Object source = event.getSource();
				if (source == editorFeaturesItem){
					boolean state = ((JCheckBoxMenuItem)source).getState();
					props.setProperty("editor_features", "" + state);
					if (state){
						editorLogin();
					} else {
						editorMenus();
					}
				}
			}
		};

		menuBar = new JMenuBar();

		JMenu fileMenu = new JMenu(TulipChain.labels.getString("file_menu"), true);
		fileMenu.setMnemonic(TulipChain.labels.getString("file_menu_key").charAt(0));
		exitItem = new JMenuItem(TulipChain.labels.getString("exit_option"), TulipChain.labels.getString("exit_option_key").charAt(0));
		exitItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F4, ActionEvent.ALT_MASK));
		exitItem.addActionListener(menuListener);
		fileMenu.add(exitItem);
		menuBar.add(fileMenu);

		JMenu editMenu = new JMenu(TulipChain.labels.getString("edit_menu"), true);
		editMenu.setMnemonic(TulipChain.labels.getString("edit_menu_key").charAt(0));
		editorFeaturesItem = new JCheckBoxMenuItem(TulipChain.labels.getString("editor_option"), props.getProperty("editor_features", "false").equals("true"));
		editorFeaturesItem.setMnemonic(TulipChain.labels.getString("editor_option_key").charAt(0));
		editorFeaturesItem.addItemListener(itemListener);
		editMenu.add(editorFeaturesItem);
		OptionsItem = new JMenuItem(TulipChain.labels.getString("options_option"), TulipChain.labels.getString("options_option_key").charAt(0));
		OptionsItem.addActionListener(menuListener);
		editMenu.add(OptionsItem);
		menuBar.add(editMenu);

		JMenu aboutMenu = new JMenu(TulipChain.labels.getString("help_menu"), true);
		aboutMenu.setMnemonic(TulipChain.labels.getString("help_menu_key").charAt(0));
		aboutItem = new JMenuItem(TulipChain.labels.getString("about_option"), TulipChain.labels.getString("about_option_key").charAt(0));
		aboutItem.addActionListener(menuListener);
		aboutMenu.add(aboutItem);
		websiteItem = new JMenuItem(TulipChain.labels.getString("website_option"), TulipChain.labels.getString("website_option_key").charAt(0));
		websiteItem.addActionListener(menuListener);
		aboutMenu.add(websiteItem);
		forumItem = new JMenuItem(TulipChain.labels.getString("forum_option"), TulipChain.labels.getString("forum_option_key").charAt(0));
		forumItem.addActionListener(menuListener);
		aboutMenu.add(forumItem);
		dmozItem = new JMenuItem(TulipChain.labels.getString("odp_option"), TulipChain.labels.getString("odp_option_key").charAt(0));
		dmozItem.addActionListener(menuListener);
		aboutMenu.add(dmozItem);
		licenseItem = new JMenuItem(TulipChain.labels.getString("license_option"), TulipChain.labels.getString("license_option_key").charAt(0));
		licenseItem.addActionListener(menuListener);
		aboutMenu.add(licenseItem);
		updateItem = new JMenuItem(TulipChain.labels.getString("update_option"), TulipChain.labels.getString("update_option_key").charAt(0));
		updateItem.addActionListener(menuListener);
		aboutMenu.add(updateItem);
		menuBar.add(aboutMenu);

		setJMenuBar(menuBar);
	}

	private JMenu catsMenu;

	private JMenuItem refreshCategories;
	private JSeparator refreshCategoriesSeparator = new JSeparator();
	private JSeparator greenbustCategoriesSeparator = new JSeparator();

	void editorMenus(){
		if (catsMenu != null){
			menuBar.remove(catsMenu);
			catsMenu = null;
		}
		if(props.getProperty("editor_features", "false").equals("true")){
			catsMenu = new JMenu(TulipChain.labels.getString("category_menu"), true);
			catsMenu.setMnemonic(TulipChain.labels.getString("category_menu_key").charAt(0));
			ActionListener catMenuListener = new ActionListener(){
				public void actionPerformed(java.awt.event.ActionEvent event){
					threadManager.selectCat(event.getActionCommand());
				}
			};
			StringTokenizer tok = new StringTokenizer(props.getProperty("editor_categories", ""), ":", false);
			while (tok.hasMoreTokens()){
				String nextTok = tok.nextToken();
				// don't add top level categories since they are easily accessible anyway.
				if (nextTok.startsWith("/")){
					nextTok = nextTok.substring(1);
				}
				if (nextTok.endsWith("/")){
					nextTok = nextTok.substring(0, nextTok.length() - 1);
				}
				if (nextTok.indexOf("/") != -1){
					JMenuItem item = new JMenuItem(nextTok);
					item.addActionListener(catMenuListener);
					catsMenu.add(item);
				}
			}
            boolean addedGreenbusts = false;
            tok = new StringTokenizer(props.getProperty("greenbust_categories", ""), ":", false);
			while (tok.hasMoreTokens()){
				String nextTok = tok.nextToken();
				// don't add top level categories since they are easily accessible anyway.
				if (nextTok.startsWith("/")){
					nextTok = nextTok.substring(1);
				}
				if (nextTok.endsWith("/")){
					nextTok = nextTok.substring(0, nextTok.length() - 1);
				}
				if (nextTok.indexOf("/") != -1){
                	if (!addedGreenbusts){
						catsMenu.add(greenbustCategoriesSeparator);
                        addedGreenbusts = true;
                    }
					JMenuItem item = new JMenuItem(nextTok);
					item.addActionListener(catMenuListener);
					catsMenu.add(item);
				}
			}
			catsMenu.add(refreshCategoriesSeparator);
			if (refreshCategories == null){
				refreshCategories = new JMenuItem(TulipChain.labels.getString("update_categories"));
				refreshCategories.addActionListener(new ActionListener(){
					public void actionPerformed(java.awt.event.ActionEvent event){
						threadManager.loadProfile();
					}
				});
			}
			catsMenu.add(refreshCategories);
			menuBar.add(catsMenu, 2);
		}
		setJMenuBar(menuBar);
	}

	private void exitRoutine(){
		Dimension d = getSize();
		props.put("window_size_x", ("" + d.width));
		props.put("window_size_y", ("" + d.height));
		Point p = getLocation();
		props.put("window_location_x", ("" + p.x));
		props.put("window_location_y", ("" + p.y));
		props.put("split_pane_divider", ("" + splitPane.getDividerLocation()));
		props.put("tree_pane_width", ("" + treeView.getSize().width));
		if (lastCatSelect != null){
			props.put("last_cat", lastCatSelect.getName());
		}
		for (int i = 0; i < table.getColumnCount(); i++) {
			TableColumn column = table.getColumnModel().getColumn(i);
			props.put("columnWidth" + i, "" + column.getWidth());
		}
		storeProperties();
		System.exit(0);
	}

	private synchronized  void editorLogin() {
		String name = props.getProperty("editor_name");
		String pass = props.getProperty("editor_pass");

		PasswordDialog passDialog = new PasswordDialog(this, TulipChain.labels.getString("editor_login"));
		if (locale != null){
			passDialog.setLocale(locale);
		}
		passDialog.setName(name);
		passDialog.setPass(pass);
		if (passDialog.showDialog()){
			name = passDialog.getName().toLowerCase();
			pass = passDialog.getPass();
			props.setProperty("editor_name", name);
			props.setProperty("editor_pass", pass);
			if (pass.equals("")){
				pass = null;
				JOptionPane.showMessageDialog(
					this,
					TulipChain.labels.getString("empty_password_message"),
					TulipChain.labels.getString("empty_password_title"),
					JOptionPane.WARNING_MESSAGE
				);
				props.remove("editor_pass");
			}
			threadManager.loadProfile();
		}
	}

	boolean canGreenbust(String cat){
    	if (!cat.endsWith("/unreviewed /")) return false;
		if (props.getProperty("editor_features", "false").equals("true")){
			StringTokenizer tok = new StringTokenizer(props.getProperty("greenbust_categories", ""), ":", false);
			while (tok.hasMoreTokens()){
				if (cat.startsWith(tok.nextToken())){
					return true;
				}
			}
		}
		return false;
	}


	boolean canEdit(String cat){
		if (props.getProperty("editor_features", "false").equals("true")){
			StringTokenizer tok = new StringTokenizer(props.getProperty("editor_categories", ""), ":", false);
			while (tok.hasMoreTokens()){
				if (cat.startsWith(tok.nextToken())){
					return true;
				}
			}
		}
		return false;
	}

	private void expandAll (DmozCat cat){
		synchronized (tree){
			tree.expandPath(new TreePath(cat.getPath()));
			for (Enumeration enum = cat.children() ; enum.hasMoreElements() ;) {
				expandAll((DmozCat)enum.nextElement());
			}
		}
	}

	private void generateReport(DmozCat cat, Properties props) throws IOException{
        if (props.getProperty("report.language", "default").equals("default")){
            reportLabels = labels;
        } else {
            Locale reportLocale = null;
            String[] locElem = StringHelper.split(props.getProperty("report.language"),"_");
			if (locElem.length >= 3) reportLocale = new Locale(locElem[0], locElem[1], locElem[2]);
			if (locElem.length == 2) reportLocale = new Locale(locElem[0], locElem[1]);
			if (locElem.length == 1) reportLocale = new Locale(locElem[0]);
            reportLabels = ResourceBundle.getBundle(
				"com.Ostermiller.TulipChain.TulipChain",
                reportLocale				
			);
        }		ReportOptions options = new ReportOptions();
		options.unchecked = (props.getProperty("report.unchecked", "false").equals("true"));
		options.ok = (props.getProperty("report.ok", "false").equals("true"));
		options.slash = (props.getProperty("report.slash", "true").equals("true"));
		options.warning = (props.getProperty("report.warning", "true").equals("true"));
		options.connectionError = (props.getProperty("report.connectionError", "true").equals("true"));
		options.error = (props.getProperty("report.error", "true").equals("true"));
		options.newWindows = (props.getProperty("report.newWindows", "false").equals("true"));
		options.ordered = (props.getProperty("report.ordered", "false").equals("true"));
		options.autoCorrect = (props.getProperty("report.autoCorrect", "false").equals("true"));
		options.showEmptyCats = (props.getProperty("report.showEmptyCats", "true").equals("true"));
		options.includeReviewed = (props.getProperty("report.includeReviewed", "true").equals("true"));
		options.includeUnreviewed = (props.getProperty("report.includeUnreviewed", "true").equals("true"));
		File reportFile = new File(
			new StringTokenizer(
				props.getProperty("report.file"),
				System.getProperty("path.separator"),
				false
			).nextToken()
		);
		File parent = reportFile.getParentFile();
		if (!parent.exists()){
			int result = JOptionPane.showConfirmDialog(
				TulipChain.this,
				MessageFormat.format(
					TulipChain.reportLabels.getString("directory_confirm_message"),
					new Object[] {
						parent.getPath()
					}
				),
				TulipChain.reportLabels.getString("directory_confirm_title"),
				JOptionPane.YES_NO_OPTION
			);
			if (result == JOptionPane.NO_OPTION){
				return;
			}
			parent.mkdirs();
		}
		if (reportFile.exists()) reportFile.delete();
		reportFile.createNewFile();
		PrintWriter out = new PrintWriter(new OutputStreamWriter(new FileOutputStream(reportFile),cat.getEncoding()));
		out.println("<html>");
		out.println("<head>");
		out.println("<meta http-equiv='content-type' content='text/html;charset=" + cat.getEncoding() + "'>");
		out.println("<title>" + MessageFormat.format(
			TulipChain.reportLabels.getString("report_title"),
			new Object[] {
				cat.getName()
			}
		) + "</title>");
		out.println("<style type=\"text/css\"><!--");
		out.println("input {");
		out.println("display:inline;");
		out.println("}");
		out.println("--></style>");
		out.println("<script language=\"javascript\" type=\"text/javascript\"><!--");
		out.println("function acConfirm(){");
		if (props.getProperty("autocorrect.noconfirm", "false").equals("true")){
			out.println("return true;");
		} else {
			out.println("return window.confirm('" + StringHelper.replace(TulipChain.reportLabels.getString("autocorrect_confirm_message"),"\n","\\n") + "');");
		}
		out.println("}");
		out.println("--></script>");

		out.println("</head>");
		out.println("<body bgcolor=\"white\">");
		out.println("<script language=\"javascript\" type=\"text/javascript\"><!--");
		out.println("if (new Date() > new Date(\"" + javascriptDateFormat.format(fourDays) + "\")){");
		out.println("document.write(\"<p><big>" + TulipChain.reportLabels.getString("report_out_of_date_warning") + "\");");
		if (options.autoCorrect){
			out.println("document.write(\"    " + TulipChain.reportLabels.getString("report_autocorrect_warning") + "\");");
		}
		out.println("document.write(\"</big></p>\");");
		out.println("document.styleSheets[0].cssRules[0].style.display=\"none\";");
		out.println("}");
		out.println("--></script>");
		if (options.autoCorrect){
			out.println("<p>" + TulipChain.reportLabels.getString("report_autocorrect_warning1") + "</p>");
		}
		reportDump(cat, options, out);
		out.println("<br clear='all'>");
		out.println("<table border='1' summary='" + TulipChain.reportLabels.getString("report_totals") + "'>");
		out.println("<tr>");
		out.println("<th bgcolor='#" + toHex(combineColors(options.REV_COLORS[0], options.STATUS_COLORS[0])) + "'>");
		out.println(TulipChain.reportLabels.getString("report_totals"));
		out.println("</th>");
		for (int i=0; i<=options.REV_TOTAL; i++){
			out.println("<th bgcolor='#" + toHex(combineColors(options.REV_COLORS[i+1], options.STATUS_COLORS[0])) + "'>");
			switch (i){
			    case 0: out.println(TulipChain.reportLabels.getString("report_reviewed")); break;
			    case 1: out.println(TulipChain.reportLabels.getString("report_unreviewed")); break;
			    case 2: out.println(TulipChain.reportLabels.getString("report_total")); break;
            }
			out.println("</th>");
		}
		for (int i=0; i<=options.STATUS_TOTAL; i++){
			out.println("<tr>");
			out.println("<th bgcolor='#" + toHex(combineColors(options.REV_COLORS[0], options.STATUS_COLORS[i+1])) + "'>");
			
            switch (i){
                case 0: out.println(TulipChain.reportLabels.getString("report_status_unknown")); break;
			    case 1: out.println(TulipChain.reportLabels.getString("report_status_ok")); break;
			    case 2: out.println(TulipChain.reportLabels.getString("report_status_slash")); break;
			    case 3: out.println(TulipChain.reportLabels.getString("report_status_warning")); break;
			    case 4: out.println(TulipChain.reportLabels.getString("report_status_connection_error")); break;
			    case 5: out.println(TulipChain.reportLabels.getString("report_status_error")); break;
			    case 6: out.println(TulipChain.reportLabels.getString("report_total")); break;
            }
			out.println("</th>");
			for (int j=0; j<=options.REV_TOTAL; j++){
				out.println("<td align=right bgcolor='#" + toHex(lighten((combineColors(options.REV_COLORS[j+1], options.STATUS_COLORS[i+1])))) + "'>");
				out.println(options.stats[i][j]);
				out.println("</td>");
			}
			out.println("</tr>");
		}
		out.println("</table>");
		out.println(
			"<br clear='all'><hr><small>" +
			MessageFormat.format(
				TulipChain.reportLabels.getString("report_generated"),
				new Object[] {
					"http://ostermiller.org/tulipchain/",
					version,
					new Date()
				}
			) +
			"<small>"
		);
		out.println("</body>");
		out.println("</html>");
		out.close();
		String url = reportFile.toURL().toString();
		Browser.displayURL(url);
	}

	private void reportDump(DmozCat cat, ReportOptions options, PrintWriter out){
	String catName = cat.getName();
		String reviewedName = null;
		StringBuffer colonName = new StringBuffer(catName.length());
		StringTokenizer tok = new StringTokenizer(catName, "/");
		while(tok.hasMoreTokens()){
			try {
				colonName.append(URLDecoder.decode(tok.nextToken(),cat.getParentEncoding()));
			} catch (java.io.UnsupportedEncodingException uee){
				colonName.append(MessageFormat.format(
					TulipChain.reportLabels.getString("unsupported_encoding"),
					new Object[] {
						cat.getParentEncoding()
					}
				));
			}
			if (tok.hasMoreTokens()){
				colonName.append(": ");
			}
		}
		if (catName.endsWith("/unreviewed /")){
			reviewedName = catName.substring(0, catName.length()-12);
		}
		boolean found = false;
		boolean headPrinted = false;
		if (options.showEmptyCats){
			 printCatHead(catName, colonName.toString(), reviewedName, options, out);
			 headPrinted = true;
		}
		for (int i=0; i < cat.getListingCount(); i++){
			DmozListing listing = cat.getListing(i);
			URLStatus s = listing.getURLwStatus();
			boolean printme = false;
			options.stats[options.STATUS_TOTAL][options.REV_TOTAL]++;
			switch (s.status){
				case URLStatus.UNKNOWN: {
					options.stats[options.UNCHECKED][options.REV_TOTAL]++;
					if (reviewedName == null){
						options.stats[options.UNCHECKED][options.REVIEWED]++;
					} else {
						options.stats[options.UNCHECKED][options.UNREVIEWED]++;
					}
					if (options.unchecked) printme = true;
					break;
				}
				case URLStatus.OK: {
					options.stats[options.OK][options.REV_TOTAL]++;
					if (reviewedName == null){
						options.stats[options.OK][options.REVIEWED]++;
					} else {
						options.stats[options.OK][options.UNREVIEWED]++;
					}
					if (options.ok) printme = true;
					break;
				}
				case URLStatus.NEEDS_SLASH: {
					options.stats[options.SLASH][options.REV_TOTAL]++;
					if (reviewedName == null){
						options.stats[options.SLASH][options.REVIEWED]++;
					} else {
						options.stats[options.SLASH][options.UNREVIEWED]++;
					}
					if (options.slash) printme = true;
					break;
				}
				case URLStatus.WARNING: {
					options.stats[options.WARNING][options.REV_TOTAL]++;
					if (reviewedName == null){
						options.stats[options.WARNING][options.REVIEWED]++;
					} else {
						options.stats[options.WARNING][options.UNREVIEWED]++;
					}
					if (options.warning) printme = true;
					break;
				}
				case URLStatus.CONNECTION_ERROR: {
					options.stats[options.CONNECTION_ERROR][options.REV_TOTAL]++;
					if (reviewedName == null){
						options.stats[options.CONNECTION_ERROR][options.REVIEWED]++;
					} else {
						options.stats[options.CONNECTION_ERROR][options.UNREVIEWED]++;
					}
					if (options.connectionError) printme = true;
					break;
				}
				case URLStatus.ERROR: {
					options.stats[options.ERROR][options.REV_TOTAL]++;
					if (reviewedName == null){
						options.stats[options.ERROR][options.REVIEWED]++;
					} else {
						options.stats[options.ERROR][options.UNREVIEWED]++;
					}
					if (options.error) printme = true;
					break;
				}
			}
			if (reviewedName == null){
				options.stats[options.STATUS_TOTAL][options.REVIEWED]++;
				if(!options.includeReviewed) printme = false;
			} else {
				options.stats[options.STATUS_TOTAL][options.UNREVIEWED]++;
				if(!options.includeUnreviewed) printme = false;
			}
			if (!headPrinted && printme){
				printCatHead(catName, colonName.toString(), reviewedName, options, out);
				headPrinted = true;
			}
			if (printme){
				found = true;
				boolean autocorrect = (options.autoCorrect && s.canAutoCorrect());
				out.print("<li>");
				if (autocorrect){
					out.print("<form onsubmit='return acConfirm();' method=post action=\"");
					if (reviewedName != null){
						out.print("http://" + getEditingHost() + "/editors/editunrev2.cgi");
					} else {
						out.print("http://" + getEditingHost() + "/editors/editurl2.cgi");
					}
					out.print("\"");
					if (options.newWindows){
						out.print(" target='_blank'");
					}
					out.print(">");
				}
				out.print("[<a href='");
				if (reviewedName != null){
					out.print("http://" + getEditingHost() + "/editors/editunrev.cgi?site=" + listing.getIDNum() + "&cat=" + reviewedName + "&chainsaw=1");
				} else {
					out.print("http://" + getEditingHost() + "/editors/editurl.cgi?url=" + URLEncodeToCatCharset(s.url,cat) + "&cat=" + catName);
				}
				out.print("'");
				if (options.newWindows){
					out.print(" target='_blank'");
				}
				out.print(">" + TulipChain.reportLabels.getString("report_edit") + "</a>] ");
				if (autocorrect){
					if (reviewedName != null){
						out.print("<input type=\"hidden\" name=\"site\" value=\"" + listing.getIDNum() + "\">");
						out.print("<input type=\"hidden\" name=\"cat\" value=\"" + reviewedName +"\">");
						out.print("<input type=\"hidden\" name=\"typecat\" value=\"" + reviewedName +"\">");
					} else {
						out.print("<input type=\"hidden\" name=\"cat\" value=\"" + catName +"\">");
					}
					String host = s.getRedirectURL().getHost();
					if (catName.startsWith("Regional/Europe/United_Kingdom/") || (host != null && (host.endsWith(".uk") || host.endsWith(".UK")))){
						out.print("<input type=\"hidden\" name=\"uksite\" value=\"on\">");
					}
					out.print("<input type=\"hidden\" name=\"url\" value=\"" + StringHelper.escapeHTML(s.url) + "\">");
					out.print("<input type=\"hidden\" name=\"newurl\" value=\"" + StringHelper.escapeHTML(s.getRedirectURL().toString()) + "\">");
					out.print("<input type=\"hidden\" name=\"newtitle\" value=\"" + StringHelper.escapeHTML(listing.getTitle()) + "\">");
					out.print("<input type=\"hidden\" name=\"newdesc\" value=\"" + StringHelper.escapeHTML(listing.getDescription()) + "\">");
					out.print("<input type=\"hidden\" name=\"newnote\" value=\"TulipChain/" + version + " Auto Correct " + StringHelper.escapeHTML(s.getDisplayStatus()) + "\">");
					if (listing.getMediaDate() != null){
						out.print("<input type=\"hidden\" name=\"date\" value=\"" + mediaDateSubmitFormat.format(listing.getMediaDate()) +"\">");
					}
					if (listing.getAgeTagKids()){
						out.print("<input type=\"hidden\" name=\"age-kids\" value=\"on\">");
					}
					if (listing.getAgeTagTeens()){
						out.print("<input type=\"hidden\" name=\"age-teen\" value=\"on\">");
					}
					if (listing.getAgeTagMatureTeens()){
						out.print("<input type=\"hidden\" name=\"age-mteen\" value=\"on\">");
					}
					out.print("<input type=\"hidden\" name=\"omit\" value=\"");
					if (reviewedName != null){
						out.print("unrev");
					} else {
						out.print("no");
					}
					out.print("\">");
					out.print("<input type=\"hidden\" name=\"submit\" value=\"Update\">");
					out.print(" <input type=\"submit\" value=\"" + TulipChain.reportLabels.getString("report_auto_correct") + "\"> ");
				}
				out.print("<span style='background-color: #");
				out.print(StringHelper.prepad(Integer.toHexString(s.getColor()), 6, '0'));
				out.print("'>[");
				out.print(s.getDisplayStatus());
				out.print("]</span> ");
				out.print("<a href='");
				out.print(StringHelper.escapeHTML(s.url));
				out.print("'");
				if (options.newWindows){
					out.print(" target='_blank'");
				}
				out.print(">");
				out.print(StringHelper.escapeHTML(listing.getTitle()));
				out.print("</a> - ");
				out.print(StringHelper.escapeHTML(listing.getDescription()));
				if (autocorrect){
					out.print("</form>");
				}
				out.println("</li>");
			}
		}
		if (headPrinted) {
			if (!found && options.showEmptyCats){
				out.println("<li>" + TulipChain.reportLabels.getString("report_nothing") + "</li>");
			}
			out.println(options.ordered?"</ol>":"</ul>");
		}
		if (cat.spidered()){
			for (Enumeration enum = cat.children() ; enum.hasMoreElements() ;) {
            	DmozCat nextCat = (DmozCat)enum.nextElement();
                if (nextCat.getEncoding().equals(cat.getEncoding())){
			   		reportDump((DmozCat)enum.nextElement(), options, out);
                } else {
                	System.err.println("Omitting category with different charset: " + nextCat.getName());
                }
			}
		}
	}

	private void printCatHead(String catName, String colonName, String reviewedName, ReportOptions options, PrintWriter out){
		out.print("<big>[<a href='");
		if (reviewedName != null){
			out.print("http://" + getEditingHost() + "/editors/editcat-unrev.cgi?cat=" + reviewedName + "&chainsaw=1");
		} else {
			out.print("http://" + getEditingHost() + "/editors/editcat.cgi?cat=" + catName);
		}
		out.print("'");
		if (options.newWindows){
			out.print(" target='_blank'");
		}
		out.print(">" + TulipChain.reportLabels.getString("report_edit") + "</a>] ");
		if (reviewedName == null){
			out.print("<a href='");
			out.print("http://" + getPublicHost() + "/" + catName);
			out.print("'");
			if (options.newWindows){
				out.print(" target='_blank'");
			}
			out.print(">");
			out.print(colonName);
			out.print("</a>");
		} else {
			out.print(colonName);
		}
		out.print("</big>");
		out.println(options.ordered?"<ol>":"<ul>");
	}

	private class ReportOptions {
		boolean unchecked = false;
		boolean ok = false;
		boolean slash = true;
		boolean warning = true;
		boolean connectionError = true;
		boolean error = true;
		boolean newWindows = false;
		boolean ordered = false;
		boolean autoCorrect = false;
		boolean showEmptyCats = true;
		boolean includeReviewed = true;
		boolean includeUnreviewed = true;


		public final static int UNCHECKED  = 0;
		public final static int OK  = 1;
		public final static int SLASH  = 2;
		public final static int WARNING  = 3;
		public final static int CONNECTION_ERROR  = 4;
		public final static int ERROR  = 5;
		public final static int STATUS_TOTAL  = 6;
		public final static int REVIEWED  = 0;
		public final static int UNREVIEWED  = 1;
		public final static int REV_TOTAL  = 2;

		int[][] stats = new int[STATUS_TOTAL+1][REV_TOTAL+1];

		public final Color[] STATUS_COLORS = {
			new Color(0xFFFFFF),
			new Color(URLStatus.getColor(URLStatus.UNKNOWN)),
			new Color(URLStatus.getColor(URLStatus.OK)),
			new Color(URLStatus.getColor(URLStatus.NEEDS_SLASH)),
			new Color(URLStatus.getColor(URLStatus.WARNING)),
			new Color(URLStatus.getColor(URLStatus.CONNECTION_ERROR)),
			new Color(URLStatus.getColor(URLStatus.ERROR)),
			new Color(0xCCCCCC),
		};

		public final Color[] REV_COLORS = {
			new Color(0xFFFFFF),
			new Color(0x9999FF),
			new Color(0xCCCCFF),
			new Color(0xCCCCCC),
		};
	}

	private static Color combineColors(Color c1, Color c2){
		return new Color(
			0xff - Math.min(
				255,
				((0xff-c1.getRed())+(0xff-c2.getRed()))
			),
			0xff - Math.min(
				255,
				((0xff-c1.getGreen())+(0xff-c2.getGreen()))
			),
			0xff - Math.min(
				255,
				((0xff-c1.getBlue())+(0xff-c2.getBlue()))
			)
		);
	}

	private static Color lighten(Color c){
		return new Color((c.getRed() + 0xff)/2,(c.getGreen() + 0xff)/2,(c.getBlue() + 0xff)/2);
	}

	private String toHex(Color c){
		return (
			StringHelper.prepad(Integer.toHexString(c.getRed()),2,'0') +
			StringHelper.prepad(Integer.toHexString(c.getGreen()),2,'0') +
			StringHelper.prepad(Integer.toHexString(c.getBlue()),2,'0')
		);
	}
}

