/*
 * TulipChain is the dmoz browser.  http://ostermiller.org/tulipchain/
 * Copyright (C) 2001-2003 Stephen Ostermiller
 * http://ostermiller.org/contact.pl?regarding=TulipChain
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * See COPYING.TXT for details.
 */

package com.Ostermiller.TulipChain;

import javax.swing.*;
import javax.swing.table.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.border.Border;
import java.net.*;
import java.io.*;
import java.util.*;
import java.text.MessageFormat;
import com.Ostermiller.util.StringHelper;
import HTTPClient.*;

class URLStatus implements Haltable, Runnable {

	public static final int UNKNOWN = 0;
	public static final int OK = 1;
	public static final int NEEDS_SLASH = 2;
	public static final int WARNING = 3;
	public static final int CONNECTION_ERROR = 4;
	public static final int ERROR = 5;

	private static final int MAX_REDIRECTS = 10;


	public String url;
	public int status = UNKNOWN;
	private String displayStatus = "";
	private DmozCat parent;
	private DmozListing listing;
	private Properties props = null;
	URL redirectURL = null;

	public URL getRedirectURL(){
		return redirectURL;
	}

	public void setProperties(Properties props){
		this.props = props;
	}

	private volatile  boolean halt = false;
	public void halt(){
		halt = true;
		Thread thread = getThread();
		if (httpConnection != null) httpConnection.stop();
		if (thread != null && thread.isAlive()) thread.interrupt();
		if (httpConnection != null) httpConnection.stop();
		if (thread != null && thread.isAlive()) thread.interrupt();
		if (httpConnection != null) httpConnection.stop();
	}

	public void setParent(DmozCat cat){
		parent = cat;
	}

	public void setListing(DmozListing listing){
		this.listing = listing;
	}

	public DmozListing getListing(){
		return this.listing;
	}

	public String getDisplayStatus(){
		return displayStatus;
	}

	public String getToolTip(){
		if (displayStatus == null || displayStatus.length() == 0){
			return null;
		}
		StringBuffer sb = new StringBuffer();
		for (int i=0; i<displayStatus.length(); i++){
			char c = displayStatus.charAt(i);
			switch (c){
				case '\r': case '\n': {
					sb.append(' ');
					break;
				}
				default: {
					sb.append(c);
				}
			}
		}
		return sb.toString();
	}

	public String toString(){
		return url;
	}

	private Thread checkThread = null;
	public void start() {
		if (checkThread == null || !checkThread.isAlive()) {
			checkThread = new Thread(this, "URLStatus");
			checkThread.start();
		}
	}

	public Thread getThread(){
		return checkThread;
	}

	public void run(){
		halt = false;
		check();
	}

	public void autoCorrect(){
		if (canAutoCorrect()){
			url = redirectURL.toString();
			status = URLStatus.OK;
			displayStatus = "";
			alertTableChange();
		}
	}

	public boolean canAutoCorrect(){
		return (
			(status == URLStatus.WARNING ||
			status == URLStatus.NEEDS_SLASH) &&
			getRedirectURL() != null
		);
	}

	private HTTPConnection httpConnection;

	public void check(){
		URL site = null;
		status = UNKNOWN;
		displayStatus = "";
		boolean redirect = true;
		String workingURL = url;
		int redirects;
		if (workingURL.toLowerCase().startsWith("file:")){
			status = ERROR;
			displayStatus += TulipChain.labels.getString("check_file_url_error");
		} else if (workingURL.toLowerCase().startsWith("https:")){
			status = UNKNOWN;
			displayStatus += TulipChain.labels.getString("check_https_url");
		} else if (workingURL.toLowerCase().startsWith("ftp:")){
			status = UNKNOWN;
			displayStatus += TulipChain.labels.getString("check_ftp_url");
		} else if (!workingURL.toLowerCase().startsWith("http:")){
			status = UNKNOWN;
			displayStatus += TulipChain.labels.getString("unknown_protocol_url");
		} else {
			String[] allRedirects = new String[MAX_REDIRECTS];
			for (redirects = 0; redirect && redirects < MAX_REDIRECTS; redirects++){
				boolean lastOK = false;
				redirect = false;
				try {
					if (site == null){
						site = new URL(workingURL);
					} else {
						site = new URL(site, workingURL);
					}
					String siteHost = site.getHost();
					if (siteHost == null || siteHost.length() == 0){
						throw new MalformedURLException();
					}
					if (halt) return;
					httpConnection = new HTTPConnection(site);
					httpConnection.setAllowUserInteraction(false);
					Class[] modules = httpConnection.getModules();
					for (int moduleIndex=0; moduleIndex<modules.length; moduleIndex++){
						String moduleName = modules[moduleIndex].getName();
						if ("HTTPClient.RedirectionModule".equals(moduleName)){
							httpConnection.removeModule(modules[moduleIndex]);
						} else {
							//System.out.println(moduleName);
						}
					}
					httpConnection.setTimeout(180000); // 3 Minutes
					Vector headers = new Vector();
					if (parent != null){
						String cat = parent.getName();
						if (cat.endsWith("/unreviewed /")){
							cat = cat.substring(0, cat.length()-12);
							cat = "http://" + TulipChain.getEditingHost() + "/editors/editcat-unrev.cgi?cat=" + cat;
						} else {
							cat = "http://" + TulipChain.getCanonicalHost() + "/" + cat;
						}
						headers.add(new NVPair("Referer", cat));
					}
					headers.add(new NVPair("User-Agent", TulipChain.getAgent()));
					String proxyAuthorization = TulipChain.getProxyAUthorization();
					if (proxyAuthorization != null){
						headers.add(new NVPair("Proxy-Authorization", proxyAuthorization));
					}
					if (halt) return;
					HTTPResponse httpResponse = httpConnection.Get(
						site.getPath(),
						(NVPair[])null,
						(NVPair[])headers.toArray(new NVPair[0])
					);
					if (halt) return;
					int response = httpResponse.getStatusCode();
					switch (response) {
						case 100: {
							status = WARNING;
							displayStatus += TulipChain.labels.getString("check_status_100");
							redirectURL = null;
							break;
						}
						case 101: {
							status = WARNING;
							displayStatus += TulipChain.labels.getString("check_status_101");
							redirectURL = null;
							break;
						}
						case 200:
						case 201:
						case 202:
						case 304: {
							lastOK = true;
							break;
						}
						case 203: {
							status = WARNING;
							displayStatus += TulipChain.labels.getString("check_status_203");
							redirectURL = null;
							break;
						}
						case 204: {
							status = ERROR;
							displayStatus += TulipChain.labels.getString("check_status_204");
							redirectURL = null;
							break;
						}
						case 205: {
							status = WARNING;
							displayStatus += TulipChain.labels.getString("check_status_205");
							redirectURL = null;
							break;
						}
						case 206: {
							status = ERROR;
							displayStatus += TulipChain.labels.getString("check_status_206");
							redirectURL = null;
							break;
						}
						case 300: {
							status = WARNING;
							displayStatus += MessageFormat.format(
								TulipChain.labels.getString("check_status_300"),
								new Object[] {
									httpResponse.getHeader("Location")
								}
							);
							redirectURL = null;
							break;
						}
						case 301: {
							workingURL = httpResponse.getHeader("Location");
							displayStatus += MessageFormat.format(
								TulipChain.labels.getString("check_status_301"),
								new Object[] {
									workingURL
								}
							);
							displayStatus += " ";
							redirectURL = new URL(site, workingURL);
							status = redirectStatus(redirectURL.toString(), status, url);
							redirect = true;
							break;
						}
						case 302: {
							workingURL = httpResponse.getHeader("Location");
							displayStatus += MessageFormat.format(
								TulipChain.labels.getString("check_status_302"),
								new Object[] {
									workingURL
								}
							);
							displayStatus += " ";
							redirectURL = new URL(site, workingURL);
							status = redirectStatus(redirectURL.toString(), status, url);
							redirect = true;
							break;
						}
						case 303: {
							workingURL = httpResponse.getHeader("Location");
							displayStatus += MessageFormat.format(
								TulipChain.labels.getString("check_status_303"),
								new Object[] {
									workingURL
								}
							);
							displayStatus += " ";
							redirectURL = new URL(site, workingURL);
							status = redirectStatus(redirectURL.toString(), status, url);
							redirect = true;
							break;
						}
						case 305: {
							status = WARNING;
							displayStatus += TulipChain.labels.getString("check_status_305");
							redirectURL = null;
							break;
						}
						case 307: {
							workingURL = httpResponse.getHeader("Location");
							redirectURL = null;
							displayStatus += MessageFormat.format(
								TulipChain.labels.getString("check_status_307"),
								new Object[] {
									workingURL
								}
							);
							displayStatus += " ";
							redirectURL = new URL(site, workingURL);
							status = redirectStatus(redirectURL.toString(), status, url);
							redirect = true;
							break;
						}
						case 400: {
							status = WARNING;
							displayStatus += TulipChain.labels.getString("check_status_400");
							redirectURL = null;
							break;
						}
						case 401: {
							status = ERROR;
							displayStatus += TulipChain.labels.getString("check_status_401");
							redirectURL = null;
							break;
						}
						case 402: {
							status = ERROR;
							displayStatus += TulipChain.labels.getString("check_status_402");
							redirectURL = null;
							break;
						}
						case 403: {
							status = ERROR;
							displayStatus += TulipChain.labels.getString("check_status_403");
							redirectURL = null;
							break;
						}
						case 404: {
							status = ERROR;
							displayStatus += TulipChain.labels.getString("check_status_404");
							redirectURL = null;
							break;
						}
						case 405: {
							status = ERROR;
							displayStatus += TulipChain.labels.getString("check_status_405");
							redirectURL = null;
							break;
						}
						case 406: {
							status = WARNING;
							displayStatus += TulipChain.labels.getString("check_status_406");
							redirectURL = null;
							break;
						}
						case 407: {
							status = WARNING;
							displayStatus += TulipChain.labels.getString("check_status_407");
							redirectURL = null;
							break;
						}
						case 408: {
							status = ERROR;
							displayStatus += TulipChain.labels.getString("check_status_408");
							redirectURL = null;
							break;
						}
						case 409: {
							status = ERROR;
							displayStatus += TulipChain.labels.getString("check_status_409");
							redirectURL = null;
							break;
						}
						case 410: {
							status = ERROR;
							displayStatus += TulipChain.labels.getString("check_status_410");
							redirectURL = null;
							break;
						}
						case 411: {
							status = WARNING;
							displayStatus += TulipChain.labels.getString("check_status_411");
							redirectURL = null;
							break;
						}
						case 412: {
							status = WARNING;
							displayStatus += TulipChain.labels.getString("check_status_412");
							redirectURL = null;
							break;
						}
						 case 413: {
							status = ERROR;
							displayStatus += TulipChain.labels.getString("check_status_413");
							redirectURL = null;
							break;
						}
						case 414: {
							status = ERROR;
							displayStatus += TulipChain.labels.getString("check_status_414");
							redirectURL = null;
							break;
						}
						case 415: {
							status = ERROR;
							displayStatus += TulipChain.labels.getString("check_status_415");
							redirectURL = null;
							break;
						}
						case 416: {
							status = ERROR;
							displayStatus += TulipChain.labels.getString("check_status_416");
							redirectURL = null;
							break;
						}
						case 417: {
							status = ERROR;
							displayStatus += TulipChain.labels.getString("check_status_417");
							redirectURL = null;
							break;
						}
						case 500: {
							status = ERROR;
							displayStatus += TulipChain.labels.getString("check_status_500");
							redirectURL = null;
							break;
						}
						case 501: {
							status = ERROR;
							displayStatus += TulipChain.labels.getString("check_status_501");
							redirectURL = null;
							break;
						}
						case 502: {
							status = ERROR;
							displayStatus += TulipChain.labels.getString("check_status_502");
							redirectURL = null;
							break;
						}
						case 503: {
							status = ERROR;
							displayStatus += TulipChain.labels.getString("check_status_503");
							redirectURL = null;
							break;
						}
						case 504: {
							status = ERROR;
							displayStatus += TulipChain.labels.getString("check_status_504");
							redirectURL = null;
							break;
						}
						case 505: {
							status = WARNING;
							displayStatus += TulipChain.labels.getString("check_status_505");
							redirectURL = null;
							break;
						}
						case 508: {
							status = WARNING;
							displayStatus += TulipChain.labels.getString("check_status_508");
							redirectURL = null;
							break;
						}
						case -1: {
							status = WARNING;
							displayStatus += MessageFormat.format(
								TulipChain.labels.getString("check_status_java_error"),
								new Object[] {
									new Integer(response)
								}
							);
							redirectURL = null;
							break;
						} default: {
							status = ERROR;
							displayStatus += MessageFormat.format(
								TulipChain.labels.getString("check_status_unknown"),
								new Object[] {
									new Integer(response)
								}
							);
							redirectURL = null;
						}
					}
					if (halt) return;
					// Modified 2002/04/29 - rpfuller@rpfuller.org
					// On newer JRE's, connection.getInputStream() causes an IOException on 404's and the like
					// This causes all 404's, etc. to be interpreted as connection errors
					// (sticking connection error on the end of the error string, giving things like "404 Not FoundConnection Error")
					InputStream in = null;
					if (lastOK) {
						in = httpResponse.getInputStream();
					}
					if (in != null && lastOK) {
						if (halt) return;
						RedirectLexer lex = new RedirectLexer(in);
						lex.setRedirectTime(Integer.parseInt(props.getProperty("redirect.meta.time", "30")));
						if (halt) return;
						String t = lex.next_token();
						if (t != null){
							workingURL = t;
							redirectURL = new URL(site, workingURL);
							if (redirectURL.toString().equals(site.toString())){
								displayStatus += TulipChain.labels.getString("check_status_refresh");								;
							} else {
								String message = TulipChain.labels.getString("check_status_unknown_redirect");
								int type = lex.getRedirectType();
								if (type == RedirectLexer.REDIRECT_TYPE_META){
									message = TulipChain.labels.getString("check_status_meta");
								} else if (type == RedirectLexer.REDIRECT_TYPE_FRAME){
									message = TulipChain.labels.getString("check_status_frame");
								}
								displayStatus += MessageFormat.format(
									message,
									new Object[] {
										workingURL
									}
								);
								displayStatus += " ";
								status = redirectStatus(redirectURL.toString(), status, url);
								redirect = true;
								lastOK = false;
							}
						}
					}
					if (lastOK) {
						if (site.getFile().equals("")){
							if (status <= OK){
								status = NEEDS_SLASH;
							}
							displayStatus += TulipChain.labels.getString("check_status_slash");								;
							redirectURL = new URL(site, "/");
							lastOK = false;
						}
					}
					if (lastOK){
						if (redirects == 0){
							status = OK;
						} else {
							if (!displayStatus.endsWith(" ")){
								displayStatus += " ";
							}
							displayStatus += TulipChain.labels.getString("check_status_200");
						}
					}
					for (int i=0; i<redirects && redirect; i++){
						if (site.toString().equals(allRedirects[i])){
							status = ERROR;
							displayStatus += TulipChain.labels.getString("check_status_recursive");
							redirect = false;
						}
					}
					allRedirects[redirects] = site.toString();
					if (halt) return;
					if (in != null) in.close();
				} catch (MalformedURLException mfx){
					status = ERROR;
					displayStatus += TulipChain.labels.getString("check_status_malformed_url");
					redirect = false;
					redirectURL = null;
				} catch (HTTPClient.ModuleException mx){
					status = CONNECTION_ERROR;
					displayStatus += mx.getMessage();
					redirect = false;
					redirectURL = null;
				} catch (UnknownHostException uhx){
					status = ERROR;
					displayStatus += TulipChain.labels.getString("check_status_no_host");
					redirect = false;
					redirectURL = null;
				} catch (NoRouteToHostException nrthx){
					status = CONNECTION_ERROR;
					displayStatus += TulipChain.labels.getString("check_status_no_route");
					redirect = false;
					redirectURL = null;
				} catch (ConnectException cx){
					status = CONNECTION_ERROR;
					displayStatus += TulipChain.labels.getString("check_status_refused");
					redirect = false;
					redirectURL = null;
				} catch (InterruptedIOException iiox){
					if (halt){
						status = UNKNOWN;
						displayStatus = "";
					} else {
						status = CONNECTION_ERROR;
						displayStatus += TulipChain.labels.getString("check_status_timeout");
					}
					redirect = false;
					redirectURL = null;
				} catch (IOException iox){
			  		if (halt){
						status = UNKNOWN;
						displayStatus = "";
					} else {
						status = CONNECTION_ERROR;
						displayStatus += TulipChain.labels.getString("check_status_connection_error");
					}
					redirect = false;
					redirectURL = null;
				}
			}
			if (redirects >= MAX_REDIRECTS){
				status = ERROR;
				displayStatus += TulipChain.labels.getString("check_status_excess_redirects");
			}
		}
		alertTableChange();
	}

	private void alertTableChange(){
		if (TulipChain.table != null &&
				(parent == null || parent == TulipChain.tableModel.getCat())){
			synchronized (TulipChain.table){
				int[] selectedRows = TulipChain.table.getSelectedRows();
				TulipChain.tableModel.fireTableDataChanged();
				for (int i=0; i<selectedRows.length; i++){
					TulipChain.table.addRowSelectionInterval(selectedRows[i], selectedRows[i]);
				}
			}
		}
	}

	private static void main(String[] args){
		URLStatus site = new URLStatus();
		site.url = args[0];
		site.check();
		System.out.println("" + site.status + " " + site.displayStatus);

	}

	public static int getColor(int status){
		switch (status){
			case URLStatus.OK: {
				return(0x00FF00);
			}
			case URLStatus.WARNING: {
				return(0xFFFF00);
			}
			case URLStatus.ERROR: {
				return(0xFF0000);
			}
			case URLStatus.NEEDS_SLASH: {
				return(0xBFFF00);
			}
			case URLStatus.CONNECTION_ERROR: {
				return(0xFFC000);
			}
			default: {
				return(0xFFFFFF);
			}
		}
	}

	public int getColor(){
		return getColor(status);
	}

	private static String[] redirectErrors = null;
	private static String[] redirectOKs = null;
	private static String savedLastUpdate = "";

	private int redirectStatus(String redirectURL, int status, String originalURL){
		if (status == WARNING || status == ERROR || status == CONNECTION_ERROR){
			return status;
		}
		if (props == null){
			return WARNING;
		}
		String lowerCaseURL = redirectURL.toLowerCase();
		String lastUpdate = props.getProperty("properties.updated", "");
		if (redirectErrors == null || redirectOKs == null || !savedLastUpdate.equals(lastUpdate)){
			redirectErrors = StringHelper.split(props.getProperty("redirect.errors", ""), ";");
			for (int i=0; i<redirectErrors.length; i++){
				redirectErrors[i] = redirectErrors[i].toLowerCase().trim();
			}
			redirectOKs = StringHelper.split(props.getProperty("redirect.oks", ""), ";");
			for (int i=0; i<redirectOKs.length; i++){
				redirectOKs[i] = redirectOKs[i].toLowerCase().trim();
			}
			savedLastUpdate = lastUpdate;
		}
		for (int i=0; i<redirectErrors.length; i++){
			if (!redirectErrors[i].equals("") && lowerCaseURL.indexOf(redirectErrors[i]) > -1){
				return ERROR;
			}
		}
		if (props.getProperty("redirect.allowDeep", "true").equals("true") &&
			redirectURL.startsWith(originalURL)){
			String diff = redirectURL.substring(originalURL.length(),redirectURL.length());
			if (!diff.equals("") && !diff.equals("/")) return OK;
		}
		for (int i=0; i<redirectOKs.length; i++){
			if (!redirectOKs[i].equals("") && lowerCaseURL.indexOf(redirectOKs[i]) > -1){
				return OK;
			}
		}
		return WARNING;
	}
}

class URLStatusRenderer extends JLabel implements TableCellRenderer {
	Border unselectedBorder = null;
	Border selectedBorder = null;
	boolean isBordered = true;

	public URLStatusRenderer(){
		this(true);
	}

	public URLStatusRenderer(boolean isBordered){
		super();
		this.isBordered = isBordered;
		setFont(getFont().deriveFont(Font.PLAIN));
	}

	public Component getTableCellRendererComponent(JTable table, Object url,
		boolean isSelected, boolean hasFocus, int row, int column) {
		setBackground(new Color(((URLStatus)url).getColor()));
		setOpaque(true);
		setText(((URLStatus)url).url);
		String toolTip = ((URLStatus)url).getToolTip();
		if (toolTip == null){
			setToolTipText(null);
		} else {
			setToolTipText(toolTip + ((row%2==0)?"":" "));
		}
		if (isBordered) {
			if (isSelected) {
				if (selectedBorder == null) {
					selectedBorder = BorderFactory.createMatteBorder(1,2,1,2,
											  table.getSelectionBackground());
				}
				setBorder(selectedBorder);
			} else {
				if (unselectedBorder == null) {
					unselectedBorder = BorderFactory.createMatteBorder(1,2,1,2,
											  table.getBackground());
				}
				setBorder(unselectedBorder);
			}
		}
		return this;
	}
}
