/*
 * TulipChain is the dmoz browser.  http://ostermiller.org/tulipchain/
 * Copyright (C) 2001 Stephen Ostermiller
 * http://ostermiller.org/contact.pl?regarding=TulipChain
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * See COPYING.TXT for details.
 */

package com.Ostermiller.TulipChain;

import com.Ostermiller.util.Browser;
import java.io.*;
import java.net.*;
import java.util.Properties;
import java.util.Date;
import java.text.MessageFormat;
import java.text.SimpleDateFormat;
import javax.swing.*;

public class UpdateCheck implements Runnable, Haltable {

	static SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy MM dd HH mm");

	private volatile  boolean halt = false;
	private boolean alertOnFail = false;
	private boolean alertAlreadySeen = false;
	public void halt(){
		halt = true;
		getThread().interrupt();
	}

	public UpdateCheck(boolean alertOnFail, boolean alertAlreadySeen){
		this.alertOnFail = alertOnFail;
		this.alertAlreadySeen = alertAlreadySeen;
	}

	public UpdateCheck(){
	}

	public void run(){
		halt = false;
		check();
	}

	public void check(){
		HttpURLConnection connection = null;
		try {
			URL url = new URL("http://ostermiller.org/TulipChain/update");
			connection = (HttpURLConnection)url.openConnection();
			if (connection instanceof HttpURLConnection){
				TulipChain.setAgent((HttpURLConnection)connection);
			}
			InputStream in = connection.getInputStream();
			Properties props = new Properties();
			props.load(in);
			System.out.println(props.getProperty(TulipChain.version, ""));
			if (!props.getProperty(TulipChain.version, "").equals("")){
				JOptionPane.showMessageDialog(
						null,
						MessageFormat.format(
							TulipChain.labels.getString("disabled_message"),
							new Object[] {
								props.getProperty(TulipChain.version, "")
							}
						),
						TulipChain.labels.getString("disabled_message_title"),
						JOptionPane.ERROR_MESSAGE
					);
				try {
					Browser.displayURL("http://ostermiller.org/tulipchain/");
				} catch (IOException ioe){
				}
				System.exit(1);
			}
			String newVersion = props.getProperty("latest", TulipChain.version);
			TulipChain.props.put("latest_check_time", dateFormat.format(new Date()));
			boolean alert = true;
			if (newVersion.equals(TulipChain.version)){
				alert = false;
			}
			if (!alertAlreadySeen &&
					!newVersion.equals(TulipChain.props.getProperty("latest", TulipChain.version))){
				alert = false;
			}
			if (alert){
				TulipChain.props.put("latest", newVersion);
				int result = JOptionPane.showConfirmDialog(
					null,
					MessageFormat.format(
						TulipChain.labels.getString("new_version_message"),
						new Object[] {
							newVersion
						}
					),
					TulipChain.labels.getString("new_version_title"),
					JOptionPane.YES_NO_OPTION
				);
				if (result == JOptionPane.NO_OPTION){
					return;
				}
				try {
					Browser.displayURL("http://ostermiller.org/tulipchain/");
				} catch (IOException ioe){
					JOptionPane.showMessageDialog(
						null,
						ioe.getMessage(),
						TulipChain.labels.getString("browser_problem_title"),
						JOptionPane.ERROR_MESSAGE
					);
				}
			} else if(alertOnFail){
				JOptionPane.showMessageDialog(
					null,
					TulipChain.labels.getString("no_new_version_message"),
					TulipChain.labels.getString("no_new_version_title"),
					JOptionPane.INFORMATION_MESSAGE
				);
			}
		} catch (IOException x){
			if(alertOnFail){
				JOptionPane.showMessageDialog(
					null,
					x.getMessage(),
					TulipChain.labels.getString("update_problem_title"),
					JOptionPane.ERROR_MESSAGE
				);
			}
		}
	}

	private Thread checkUpdateThread = null;
	public void start() {
		if (checkUpdateThread == null || !checkUpdateThread.isAlive()) {
			checkUpdateThread = new Thread(this, "checkUpdate");
			checkUpdateThread.start();
		}
	}

	public Thread getThread(){
		return checkUpdateThread;
	}
}
